﻿  ; Check if .prefs exists and is valid, displaying a pop-up warning in #False
  ; Notice to change the version number of the .prefs in both procedures when we change the version number of .prefs
  ; This is a duplicate of the load prefs code, just to be displayed after the GUI is designed
  ; 
  ; V1.0 - 14/DEC/2016
  ;        6/JAN/2017
  ; V1.1 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ;
  Procedure check_prefs()
    
    ; Check if .PREFS file exists
    If FileSize(main_path$+"ptg3.prefs")<=0
      ProcedureReturn #False
    EndIf    
    
    ; Attempt to load .PREFS file
    ReadFile(1,main_path$+"ptg3.prefs")
    t$=ReadString(1,#PB_UTF8)
    If t$<>prefs_file_version$
      MessageRequester("Error", "The .prefs file is invalid."+#CRLF$+#CRLF$+"Maybe it is from a different version of"+#CRLF$+"Proofing Tool GUI.",#PB_MessageRequester_Error)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, ".prefs file invalid.")
      CloseFile(1)
      ProcedureReturn #False
    EndIf
    
    ProcedureReturn #True
    
  EndProcedure
  
  
  
  
  ; Loads the preferences
  ; It returns #True = Success or #False = Failed
  ; This is a duplicate of the check prefs code, just to load before the GUI is designed
  ;
  ; V1.0 - 29/SEP/2016
  ; V1.1 - 5/OCT/2016
  ;        o Improved a bit the code.
  ; V1.2 - 14/DEC/2016
  ;        o Updated load/save prefs
  ; V1.3 - 6/JAN/2017
  ;        o Now makes uses of ".AFF Aid" 
  ; V1.4 - 7/FEB/2018
  ;        o Now loads "Number separator chr"
  ; V1.5 - 15/FEB/2018
  ;        20/FEB/2018
  ;        o Improved the window size handling
  ; V1.6 - 9/MAY/2018
  ;        o Coding of "Check for Updates"  
  ; V1.7 - 11/JUL/2018
  ;        o Prefs: Now allows to set colour for "Primary", "Suffix" and "Prefix"   
  ;
  Procedure load_preferences()
    
    ; Check if .PREFS file exists
    If FileSize(main_path$+"ptg3.prefs")<=0
      ProcedureReturn #False
    EndIf    
    
    ; Attempt to load .PREFS file
    ReadFile(1,main_path$+"ptg3.prefs")
    t$=ReadString(1,#PB_UTF8)
    If t$<>prefs_file_version$
      CloseFile(1)
      ProcedureReturn #False
    EndIf
    
    ; Set resolution
    window_resolution$=ReadString(1,#PB_UTF8)
    desktop_resolution$=ReadString(1,#PB_UTF8)
    window_resolution_max$=ReadString(1,#PB_UTF8)
    
    ; Set number of lines
    number_of_lines=ReadLong(1)
    
    ; Set number of pixel plus
    pixels_plus=ReadLong(1)
    
    ; Set LanguageTool number of lines
    number_of_lines_languagetool=ReadLong(1)
    
    ; Set .AFF aid language
    aff_aid_language$=ReadString(1,#PB_UTF8) ; Now it has the filename
    
    ; Number separator chr
    number_separator_chr$=ReadString(1,#PB_UTF8)
    
    ; Colours for dictionary editor - 11/JUL/2018
    affix_colour_primary=ReadLong(1)
    affix_colour_suffix=ReadLong(1)
    affix_colour_prefix=ReadLong(1)
    
    ; Time of last check for updates
    check_for_updates$=ReadString(1,#PB_UTF8)
    last_check_updates_time.q=ReadQuad(1)
    
    ; Close file
    CloseFile(1)
  
    ; Return success
    ProcedureReturn #True
    
  EndProcedure
  
  
  
  
  ; Saves the Preferences, called from the Preferences Editor PLUS after silent automatic checking for updates
  ;
  ; V1.0 - 9/MAY/2018
  ; V1.1 - 11/JUL/2018
  ;        o Prefs: Now allows to set colour for "Primary", "Suffix" and "Prefix"   
  ;
  Procedure save_preferences()
    
    ; Delete old .prefs file    
    If FileSize(main_path$+"ptg3.prefs")>=0
      DeleteFile(main_path$+"ptg3.prefs")
    EndIf     
    
    
    ; Attempt to save .PREFS file
    OpenFile(1,main_path$+"ptg3.prefs")
                  
      ; PREFS version
      WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)

      ; Set resolution
      WriteString(1,window_resolution$+#LF$,#PB_UTF8)
      WriteString(1,desktop_resolution$+#LF$,#PB_UTF8)
      WriteString(1,window_resolution_max$+#LF$,#PB_UTF8)
    
      ; Set number of lines
      WriteLong(1,number_of_lines)
                    
      ; Set number of pixel plus
      WriteLong(1,pixels_plus)
                    
      ; Set LanguageTool number of lines
      WriteLong(1,number_of_lines_languagetool)
                    
      ; Language of .AFF Aid
      WriteString(1,aff_aid_language$+#LF$,#PB_UTF8)
                    
      ; Number separator chr
      WriteString(1,number_separator_chr$+#LF$,#PB_UTF8)
      
      ; Colours for dictionary editor - 11/JUL/2018
      WriteLong(1,affix_colour_primary)
      WriteLong(1,affix_colour_suffix)
      WriteLong(1,affix_colour_prefix)
                          
      ; Last time check for updates
      WriteString(1,check_for_updates$+#LF$,#PB_UTF8)
      WriteQuad(1,last_check_updates_time.q)
                    
    CloseFile(1)
    
    
  EndProcedure
  
  
  
  
  ; Now it allows "Custom" ".AFF Aids" that reads from the file: "custom_aff_aid.LANG.txt"+"custom_aff_aid.LANG.png"
  ; Notice that the flags are stored in number 20000+
  ; (30)=EN_GB (BUILT IN)
  ;
  ; V1.0 - 7/JAN/2017
  ; V1.1 - 19/MAR/2017
  ;        o PREFS now accept external files with rules and flags
  ; V1.2 - 21/MAR/2017  
  ;        o If no image is found for a file in the custom Aff aid files, there is no attempt to load/use the flag
  ;
  Procedure load_all_aff_aid()
    
    
    ; Current count 0 = English GB > Image = 30 > filename$="*****"
    aff_aid_language_array$(0)="eng-GB:english:English GB"+#LF$+"30"+#LF$+"*****"
    
    
    ; Get the files and the rules
    If ExamineDirectory(#DIRECTORY,main_path$+"custom_aff_aid","*.txt")
      current_count=1
      While NextDirectoryEntry(#DIRECTORY)
        If DirectoryEntryType(#DIRECTORY)=#PB_DirectoryEntry_File
          
          ; Check if file with rules exist
          If DirectoryEntrySize(#DIRECTORY)>0
            
            ; Load the rules, whose first line is the language name
            ReDim aff_aid_language_array_rules$(current_count)
            OpenFile(1,main_path$+"custom_aff_aid/"+DirectoryEntryName(#DIRECTORY))
              aff_aid_language_array_rules$(current_count)=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)  
            CloseFile(1)

            ; Replace Windows/Mac EOL with Linux ones
            aff_aid_language_array_rules$(current_count)=ReplaceString(aff_aid_language_array_rules$(current_count),#CRLF$,#LF$)
            aff_aid_language_array_rules$(current_count)=ReplaceString(aff_aid_language_array_rules$(current_count),#LFCR$,#LF$)
            
            ; Load the flag image
            If FileSize(main_path$+"custom_aff_aid/"+Left(DirectoryEntryName(#DIRECTORY),Len(DirectoryEntryName(#DIRECTORY))-3)+"png")<>-1
              LoadImage(20000+current_count,main_path$+"custom_aff_aid/"+Left(DirectoryEntryName(#DIRECTORY),Len(DirectoryEntryName(#DIRECTORY))-3)+"png")
            EndIf
            
            ; Create the entry that holds the LANGUAGE_NAME + IMAGE_NUMBER + FILENAME
            ReDim aff_aid_language_array$(current_count)
            aff_aid_language_array$(current_count)=StringField(aff_aid_language_array_rules$(current_count),1,#LF$)+#LF$+Str(20000+current_count)+#LF$+DirectoryEntryName(#DIRECTORY)
            
            ; Increase the current count
            current_count+1
            
          EndIf    
        EndIf
      Wend
      current_count-1 ; To give the exact number of files processed NOT counting with the GB one which is ID = 0
      
      ; Sort the languages
      SortArray(aff_aid_language_array$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
      
    Else
      
      ProcedureReturn #False      
      
    EndIf
    
    
  EndProcedure
  
  
  
  
  ; Set the Preferences, for now the number of lines in the ListIconGadget
  ;
  ; V1.0 - 28/SEP/2016
  ;        29/SEP/2016
  ; V1.1 - 12/DEC/2016
  ;        13/DEC/2016
  ;        o Added resolution+LanguageTool number of lines between rules to Prefs
  ;        o When the pop-up of the first time settings prefs appears, it shows an information tip.
  ; V1.2 - 14/DEC/2016
  ;        o Updated load/save prefs   
  ;        o Changing the resolution in Prefs remembers the word on the top for each tab
  ; V1.3 - 20/DEC/2016
  ;        o Fix: Exiting the PREFS after changing the resolution no longer always jump to the fist tab
  ; V1.4 - 6/JAN/2017
  ;        o Now makes uses of ".AFF Aid"
  ; V1.5 - 7/JAN/2017
  ;        o Now it allows a "Custom" ".AFF Aid" that reads from the file: "custom_aff_aid.txt" 
  ;        o Added checkboxes to the lines so that they take the real space
  ; V1.6 - 11/MAR/2017
  ;        o Changed the buttons height to 25. 
  ; V1.7 - 19/MAR/2017
  ;        o PREFS now accept external files with rules and flags
  ; V1.8 - 21/MAR/2017  
  ;        o If no image is found for a file in the custom Aff aid files, there is no attempt to load/use the flag
  ; V1.9 - 4/APR/2017
  ;        o Fix: Changing the GUI resolution in the PREFS would reset the .AFF EditorGadget
  ; V2.0 - 26/JAN/2018
  ;        o Now it reds the gadget if invalid minimum number of lines in LanguageTool
  ; V2.1 - 7/FEB/2018
  ;        o ComboBox with higher height (+6 pixels) for better Ubuntu look.
  ;        o Now tip appears all the time
  ;        o New option: "Number separator chr"
  ; V2.2 - 13/FEB/2018
  ;        o Added new resolutions to the Prefs.
  ;        o Use of Plus Pixels in Prefs + GUI
  ; V2.3 - 15/FEB/2018  
  ;        o Improved the window size handling as well as a "Maximise Window (WxH)"
  ; V2.4 - 20/FEB/2018  
  ;        o Fix: Get maximum window on PREFS now works in Linux
  ;        o Added +10 pixels to windows resolutions text to fit in Linux
  ; V2.5 - 20/MAR/2018
  ;        26/MAR/2018
  ;        o Cleaned the code
  ; V2.6 - 28/MAR/2018
  ;        o Support for Hyphenation
  ; V2.7 - 19/APR/2018
  ;        o Fix: Changing resolution in the Preferences would reset each tab filename to "n/a" and erase the Hyphenation rules
  ; V2.8 - 9/MAY/2018
  ;        o Coding of "Check for Updates"
  ; V2.9 - 11/JUL/2018
  ;        o Prefs: Now allows to set colour for "Primary", "Suffix" and "Prefix"   
  ; V3.0 - 25/JUL/2018
  ;        o Cleaned the code by using a function to turn the gadgets background to Red/White on errors.
  ;
  Procedure preferences()
    
    ; Get maximum window size from an invisible window
    ExamineDesktops()
    t$=Str(DesktopWidth(0))+"x"+Str(DesktopHeight(0))
    If t$<>desktop_resolution$
      desktop_resolution$=t$
      menu_height=MenuHeight()
      If OpenWindow(#WINDOW_PREFERENCES_MAX_SIZE,0,0,10,10,"Getting Maximum Window Size",#PB_Window_Maximize)=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
      window_resolution_max$=Str(WindowWidth(#WINDOW_PREFERENCES_MAX_SIZE))+"x"+Str(WindowHeight(#WINDOW_PREFERENCES_MAX_SIZE)-StatusBarHeight(#WINDOW_MAIN_STATUS_BAR)-menu_height)
      CloseWindow(#WINDOW_PREFERENCES_MAX_SIZE)    
    EndIf
    
    ; Open the window
    If OpenWindow(#WINDOW_PREFERENCES,0,0,ImageWidth(28)+10-30-15+300-100-150-150-100+(800-640)+640-10,10+30+200-50-20-20+100+100+200-30-30-30-40+10-10-5+get_extra_height(100),"Preferences",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    DisableWindow(#WINDOW_MAIN,#True)
    
    ; Store old variables
    old_window_resolution$=window_resolution$
    old_number_of_lines=number_of_lines
    old_pixels_plus=pixels_plus
    old_number_of_lines_languagetool=number_of_lines_languagetool
    old_aff_aid_language$=aff_aid_language$
    old_number_separator_chr$=number_separator_chr$
    old_check_for_updates$=check_for_updates$
    old_last_check_updates_time.q=last_check_updates_time.q
    top_of_listicongadget_dictionary=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
    top_of_listicongadget_thesaurus=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
    top_of_listicongadget_autocorrect=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
    old_affix_colour_primary=affix_colour_primary
    old_affix_colour_suffix=affix_colour_suffix
    old_affix_colour_prefix=affix_colour_prefix
    
    
    x=5
    y=10
    ListIconGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,x,y,200+20,GadgetHeight(#PANEL_BOX_DICTIONARY),"# Visible lines (OS specific)",1000,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection|#PB_ListIcon_CheckBoxes)
    
    
    y+GadgetHeight(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+10+5+5
    
    y-pixels_plus
    
    ; SpinGadget with the number of lines
    SpinGadget(#SPIN_WINDOW_PREFERENCES_LINES,x,y,60-8,20+2,1,50,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_LINES,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5,y+2,100,20+2+1,"Visible lines")
    v=number_of_lines
    SetGadgetState(#SPIN_WINDOW_PREFERENCES_LINES,v)
    SetGadgetText(#SPIN_WINDOW_PREFERENCES_LINES,Str(v))
    
    
    ; SpinGadget Pixels Plus
    SpinGadget(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5+GadgetWidth(#TEXT_WINDOW_PREFERENCES_LINES),y,60-8,20+2,-10,10,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_PLUSPIXELS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5+GadgetWidth(#TEXT_WINDOW_PREFERENCES_LINES)+5+GadgetWidth(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS)+5,y+2,100,20+2+1,"Plus Pixels")
    v=pixels_plus
    SetGadgetState(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,v)
    SetGadgetText(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,Str(v))         
    
    ; Window resolution
    x=GadgetX(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+GadgetWidth(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+10+15
    y=GadgetY(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)
    FrameGadget(#FRAME_WINDOW_PREFERENCES_RESOLUTION,x,y,200+10+10+15+10,200-100-40+10+20+20+2+1,"Resolution")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x600,x+10,y+10+5+2,100-10-10+10,20+2+1,"1024×600")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600,x+10,y+10+20+2+1+5+2,100-10-10+10,20+2+1,"1280×600")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600,x+10,y+10+20+2+1+5+2+20+2+1,100-10-10+10,20+2+1,"1440×600")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE,x+10,y+10+20+2+1+5+2+20+2+1+20+2+1,100-10-10+100+50,20+2+1,"Maximise Window ("+ReplaceString(window_resolution_max$,"x","×")+")")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x768,x+10+100,y+10+5+2,100-10-10+10,20+2+1,"1024×768")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768,x+10+100,y+10+20+2+1+5+2,100-10-10+10,20+2+1,"1280×768")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768,x+10+100,y+10+20+2+1+5+2+20+2+1,100-10-10+10,20+2+1,"1440×768")        
    flag=#False
    If window_resolution$="1024x600" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x600,#True)
      flag=#True
    EndIf
    If window_resolution$="1280x600" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600,#True)
      flag=#True
    EndIf           
    If window_resolution$="1440x600" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600,#True)
      flag=#True      
    EndIf
    If window_resolution$="1024x768" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x768,#True)
      flag=#True      
    EndIf
    If window_resolution$="1280x768" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768,#True)
      flag=#True
    EndIf
    If window_resolution$="1440x768" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768,#True)
      flag=#True
    EndIf    
    If flag=#False
      window_resolution$=window_resolution_max$
      SetGadgetState(#OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE,#True)
    EndIf
    
    
    ; LanguageTool - Minimum lines between rules
    y+GadgetHeight(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_LANGUAGETOOL,x,y,200+10+10+15+10,100-40-3,"LanguageTool")
    SpinGadget(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,x+10,y+10+5+2+3+2,60-8,20+2,1,50,#PB_Spin_Numeric)
    TextGadget(#TEXT_WINDOW_PREFERENCES_LANGUAGETOOL,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL)+5+5+5,GadgetY(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL)+2,200,20+2+1,"Minimum lines between rules")
    v=number_of_lines_languagetool
    SetGadgetState(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,v)
    SetGadgetText(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,Str(v))
    
    
    ; .AFF Aid combobox
    y+GadgetHeight(#FRAME_WINDOW_PREFERENCES_LANGUAGETOOL)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_AFF_AID,x,y,200+10+10+15+10,100-40-3,".AFF Aid Language")    
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,x+10,y+16+4,100+150-10-10,20+2+6,#PB_ComboBox_Image)
    AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,-1,"n/a")
    current_aff_aid_language=0
    image=0
    For f=0 To ArraySize(aff_aid_language_array$())
      If StringField(aff_aid_language_array$(f),3,#LF$)<>"" : image=Val(StringField(aff_aid_language_array$(f),2,#LF$)) : EndIf
      If IsImage(image)
        AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,-1,StringField(aff_aid_language_array$(f),1,#LF$),ImageID(image))
      Else
        AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,-1,StringField(aff_aid_language_array$(f),1,#LF$))
      EndIf
      If aff_aid_language$=StringField(aff_aid_language_array$(f),3,#LF$) : current_aff_aid_language=f+1 : EndIf  
    Next f
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,current_aff_aid_language)
    
    
    ; Number separator chr
    y+GadgetHeight(#FRAME_WINDOW_PREFERENCES_AFF_AID)+10
    FrameGadget(#FRAME_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,x,y,200+10+10+15+10,100-40-3,"Number separator character")      
    StringGadget(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,x+10,y+16+4,16+2,20+3,number_separator_chr$)
    SetGadgetAttribute(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,#PB_String_MaximumLength,1)
    TextGadget(#TEXT_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,x+10+GadgetWidth(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR)+5,y+16+4+2,200,20+2+1,"Preview:"+FormatNumber(1000000,0,".",number_separator_chr$))
    GadgetToolTip(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,"One character only. It can be blank.")
    
    
    ; Add the SAVE, OKAY and CANCEL buttons
    y=GadgetY(#SPIN_WINDOW_PREFERENCES_LINES)+GadgetHeight(#SPIN_WINDOW_PREFERENCES_LINES)+10+5-5
    x=GadgetX(#SPIN_WINDOW_PREFERENCES_LINES)+20+10+5+30+30+ImageWidth(28)/2+10
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_SAVE,x,y,90,25,"Save")
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_OKAY,x+100,y,90,25,"Okay")
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_CANCEL,x+100+100,y,90,25,"Cancel")    
    
    
    ; Populate the ListIconGadget
    For f=1 To 50
      AddGadgetItem(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,-1,"Line "+Str(f))
    Next f
    
    
    ; Check for updates
    x=GadgetX(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+GadgetWidth(#FRAME_WINDOW_PREFERENCES_RESOLUTION)+10+15
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_RESOLUTION)
    FrameGadget(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES,x,y,200+10+10+15+10+10+10+3,200-100-40+10+20+20+2+1+40+10+10,"Check for Updates")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL,x+10,y+10+5+2,100-10-10+10,20+2+1,"Manual")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_DAILY,x+10,y+10+20+2+1+5+2,100-10-10+10,20+2+1,"Daily")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_WEEKLY,x+10,y+10+20+2+1+5+2+20+2+1,100-10-10+10,20+2+1,"Weekly")
    OptionGadget(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY,x+10,y+10+20+2+1+5+2+20+2+1+20+2+1,100-10-10+100+50,20+2+1,"Monthly")
    y+2
    TextGadget(#TEXT_WINDOW_PREFERENCES_CHECKUPDATES_LASTCHECKED,x+10,y+10+20+2+1+5+2+20+2+1+20+2+1+20+2+1,100-10-10+100+50,20+2+1,"Last checked:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",last_check_updates_time.q))
    y+2
    ButtonGadget(#BUTTON_WINDOW_PREFERENCES_CHECKUPDATES,x+10,y+10+20+2+1+5+2+20+2+1+20+2+1+20+2+1+20+2+1,90,25,"Check")
    If check_for_updates$="manual" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL,#True) : EndIf
    If check_for_updates$="daily" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_DAILY,#True) : EndIf
    If check_for_updates$="weekly" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_WEEKLY,#True) : EndIf
    If check_for_updates$="monthly" : SetGadgetState(#OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY,#True) : EndIf
    
    
    ; Colours for dictionary editor - 11/JUL/2018
    y=GadgetY(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES)+10
    x=GadgetX(#FRAME_WINDOW_PREFERENCES_CHECKUPDATES)
    FrameGadget(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY,x,y,200+10+10+15+10+10+10+3,-2-35-8+200-100-40+10+20+20+2+1+40+10+10,"Affix Colours")
    y+10+5+5
    x+10
    y+2
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,x,y,90-15-20,25,"Primary ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,x+90+5-15-20,y,50,20+2+6,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY,affix_colour_primary)
    y+25+5
    y+2
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,x,y,90-15-20,25,"Suffix ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,x+90+5-15-20,y,50,20+2+6,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX,affix_colour_suffix)
    y+25+5
    y+2
    TextGadget(#TEXT_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,x,y,90-15-20,25,"Prefix ",#PB_Text_Right)
    ComboBoxGadget(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,x+90+5-15-20,y,50,20+2+6,#PB_ComboBox_Image)
    For f=0 To 7
      AddGadgetItem(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,-1,"",ImageID(61+f))
    Next f    
    SetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX,affix_colour_prefix)    
    
    
    ; Display the tip
    ImageGadget(#IMAGE_WINDOW_PREFERENCES_TIP,WindowWidth(#WINDOW_PREFERENCES)-ImageWidth(28)-10,GadgetY(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)+GadgetHeight(#FRAME_WINDOW_PREFERENCES_COLOURS_DICTIONARY)+20-10-2,244,80,ImageID(28))
        
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_PREFERENCES,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_PREFERENCES,#PB_Shortcut_Return,1003)  
    
    
    okay=#False
    cancel=#False
    Repeat
          
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu) or <ESC> pressed
      If event=#PB_Event_CloseWindow Or (event=#PB_Event_Menu And EventMenu()=1000)
        cancel=#True     
      EndIf     
      
      
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()      
              
              ; Save button
              Case #BUTTON_WINDOW_PREFERENCES_SAVE
                Debug "Pressed SAVE"
                
                flag=#False
                If InvalidValue(#WINDOW_PREFERENCES,#SPIN_WINDOW_PREFERENCES_LINES,1,50,"The number of lines must be in the range 1:50",flag) And flag=#False
                  flag=#True
                EndIf        
                
                
                If flag=#False
                  save_preferences()                                       
                EndIf
                 
                 
              ; Okay button
              Case #BUTTON_WINDOW_PREFERENCES_OKAY
                Debug "Pressed OKAY"
                flag=#False
                If InvalidValue(#WINDOW_PREFERENCES,#SPIN_WINDOW_PREFERENCES_LINES,1,50,"The number of lines must be in the range 1:50",flag) And flag=#False
                  flag=#True
                EndIf                   
                If InvalidValue(#WINDOW_PREFERENCES,#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,1,50,"The number of lines must be in the range 1:50",flag) And flag=#False
                  flag=#True
                EndIf                      
                If InvalidValue(#WINDOW_PREFERENCES,#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,-10,10,"The number of Pixels Plus must be in the range -10:10",flag) And flag=#False
                  flag=#True
                EndIf                               
                If flag=#False : okay=#True : EndIf
                
                
              ; Cancel button
              Case #BUTTON_WINDOW_PREFERENCES_CANCEL
                Debug "Pressed CANCEL"
                cancel=#True
                
                
              ; Spin Gadget NUMBER OF VISIBLE LINES
              Case #SPIN_WINDOW_PREFERENCES_LINES
                If EventType()=#PB_EventType_Change Or EventType()=#PB_EventType_Up Or EventType()=#PB_EventType_Down
                  Debug "Pressed SPIN"                  
                  
                  number_of_lines=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_LINES,1,50)
      
                EndIf                 
                
                
              ; Double-click
              Case #LISTICONGADGET_WINDOW_PREFERENCES_LINES
                If EventType()=#PB_EventType_LeftDoubleClick
                  Debug "Double-click on ListIconGadget"
                  number_of_lines=GetGadgetState(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+1
                  SetGadgetState(#SPIN_WINDOW_PREFERENCES_LINES,number_of_lines)
                  SetGadgetText(#SPIN_WINDOW_PREFERENCES_LINES,Str(number_of_lines))
                EndIf
                
                
              ; Spin Gadget PIXELS PLUS 
              Case #SPIN_WINDOW_PREFERENCES_PLUSPIXELS
                If EventType()=#PB_EventType_Change Or EventType()=#PB_EventType_Up Or EventType()=#PB_EventType_Down
                  Debug "Pressed SPIN Pixels Plus"                  
            
                  pixels_plus=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,-10,10)
                     
                  If GetGadgetColor(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,#PB_Gadget_BackColor)=-1

                    ; Resize ListIconGadget
                    ResizeGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,GadgetX(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetY(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetWidth(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),300-30+5+40+get_extra_height(100)+pixels_plus)
                      
                  EndIf                      
                         
                EndIf                 
                
                
              ; Resolution of window
              Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x600 To #OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768
                 
                Select EventGadget()
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x600
                    window_resolution$="1024x600"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x600
                    window_resolution$="1280x600"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x600
                    window_resolution$="1440x600"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1024x768
                    window_resolution$="1024x768"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1280x768
                    window_resolution$="1280x768"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_1440x768
                    window_resolution$="1440x768"
                    
                  Case #OPTION_WINDOW_PREFERENCES_RESOLUTION_MAX_WINDOW_SIZE
                    window_resolution$=window_resolution_max$
                    
                EndSelect
                
                ; Resize window
                ResizeWindow(#WINDOW_PREFERENCES,WindowX(#WINDOW_PREFERENCES),WindowY(#WINDOW_PREFERENCES),WindowWidth(#WINDOW_PREFERENCES),10+30+200-50-20-20+100+100+200-30-30-30-40+10-10-5+get_extra_height(100))
                
                ; Resize ListIconGadget
                ResizeGadget(#LISTICONGADGET_WINDOW_PREFERENCES_LINES,GadgetX(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetY(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),GadgetWidth(#LISTICONGADGET_WINDOW_PREFERENCES_LINES),300-30+5+40+get_extra_height(100)+pixels_plus)
                
                ; Move # of lines
                x=5
                y=10+GadgetHeight(#LISTICONGADGET_WINDOW_PREFERENCES_LINES)+10+5+5
                y-pixels_plus
                ; SpinGadget with the number of lines
                ResizeGadget(#SPIN_WINDOW_PREFERENCES_LINES,x,y,60-8,20+2)
                ResizeGadget(#TEXT_WINDOW_PREFERENCES_LINES,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5,y+2,100,20+2+1)                
                ; Move Pixels Plus
                ; SpinGadget Pixels Plus
                ResizeGadget(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5+GadgetWidth(#TEXT_WINDOW_PREFERENCES_LINES),y,60-8,20+2)
                ResizeGadget(#TEXT_WINDOW_PREFERENCES_PLUSPIXELS,x+GadgetWidth(#SPIN_WINDOW_PREFERENCES_LINES)+5+GadgetWidth(#TEXT_WINDOW_PREFERENCES_LINES)+5+GadgetWidth(#SPIN_WINDOW_PREFERENCES_PLUSPIXELS)+5,y+2,100,20+2+1)           
                
                ; Move Save, Okay and Cancel
                ; Add the SAVE, OKAY and CANCEL buttons
                y=GadgetY(#SPIN_WINDOW_PREFERENCES_LINES)+GadgetHeight(#SPIN_WINDOW_PREFERENCES_LINES)+10+5-5
                x=GadgetX(#SPIN_WINDOW_PREFERENCES_LINES)+20+10+5+30+30+ImageWidth(28)/2+10 
                ResizeGadget(#BUTTON_WINDOW_PREFERENCES_SAVE,x,y,90,25)              
                ResizeGadget(#BUTTON_WINDOW_PREFERENCES_OKAY,x+100,y,90,25)
                ResizeGadget(#BUTTON_WINDOW_PREFERENCES_CANCEL,x+100+100,y,90,25)
                
                
              ; Spin Gadget - LanguageTool - Minimum lines between rules
              Case #SPIN_WINDOW_PREFERENCES_LANGUAGETOOL
                If EventType()=#PB_EventType_Change Or EventType()=#PB_EventType_Up Or EventType()=#PB_EventType_Down
                  Debug "Pressed SPIN"                  
            
                  number_of_lines_languagetool=ChangeGadgetColorRed(#SPIN_WINDOW_PREFERENCES_LANGUAGETOOL,1,50)
                         
                EndIf                 
                
                
              ; Change the ComboBox of the .AFF Aid country
              Case #COMBOBOX_WINDOW_PREFERENCES_AFF_AID
                If EventType()=#PB_EventType_Change
                  Debug "Changed the .AFF Aid country"
                  
;                   aff_aid_language$=GetGadgetItemText(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID,GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID))
                  If GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID)=0
                    aff_aid_language$=""
                  Else
                    aff_aid_language$=StringField(aff_aid_language_array$(GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_AFF_AID)-1),3,#LF$)
                  EndIf
                  
                EndIf  
                
                
              ; Change the Number separator chr
              Case #STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR
                If EventType()=#PB_EventType_Change
                  Debug "Changed the Number separator chr"
                  number_separator_chr$=GetGadgetText(#STRING_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR)
                  SetGadgetText(#TEXT_WINDOW_PREFERENCES_AFF_NUMBER_SEPARATOR_CHR,"Preview:"+FormatNumber(1000000,0,".",number_separator_chr$))
                EndIf                  
                
                
              ; CHECK button
              Case #BUTTON_WINDOW_PREFERENCES_CHECKUPDATES
                  check_for_updates(#False)
                  last_check_updates_time.q=Date()
                  SetGadgetText(#TEXT_WINDOW_PREFERENCES_CHECKUPDATES_LASTCHECKED,"Last checked:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",last_check_updates_time.q))              
               
                
              ; Check for updates
              Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL To #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY
                  
                  Select EventGadget()
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MANUAL
                      check_for_updates$="manual"
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_DAILY
                      check_for_updates$="daily"
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_WEEKLY
                      check_for_updates$="weekly"
                      
                    Case #OPTION_WINDOW_PREFERENCES_CHECKUPDATES_MONTHLY
                      check_for_updates$="monthly"
                      
                  EndSelect 
                  
                  
            ; Affixes colours
            ; Primary
            Case #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY
              affix_colour_primary=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PRIMARY)
              
            ; Suffix                
            Case  #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX
              affix_colour_suffix=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_SUFFIX)    
              
            ;Prefix
            Case #COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX
              affix_colour_prefix=GetGadgetState(#COMBOBOX_WINDOW_PREFERENCES_COLOURS_DICTIONARY_PREFIX) 
                  
                  
                   
          EndSelect  
                  
              
      EndSelect
      
      
    Until okay=#True Or cancel=#True
    
    
    ; Pressed CANCEL
    ; Restore old variables
    If cancel=#True
      window_resolution$=old_window_resolution$
      number_of_lines=old_number_of_lines
      pixels_plus=old_pixels_plus
      number_of_lines_languagetool=old_number_of_lines_languagetool    
      aff_aid_language$=old_aff_aid_language$      
      number_separator_chr$=old_number_separator_chr$
      check_for_updates$=old_check_for_updates$
      last_check_updates_time.q=old_last_check_updates_time.q
      affix_colour_primary=old_affix_colour_primary
      affix_colour_suffix=old_affix_colour_suffix
      affix_colour_prefix=old_affix_colour_prefix
    EndIf
    
    
    ; Pressed OKAY
    If okay=#True
    
      If window_resolution$<>old_window_resolution$ Or old_pixels_plus<>pixels_plus
        current_tab=GetGadgetState(#PANEL_MAIN)
        t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
        t1$=GetGadgetText(#EDITORGADGET_PANEL_HYPHENATOR_DIC)
        t2$=GetGadgetText(#STRING_PANEL_HYPHENATOR_WORD)
        t3$=""
        If CountGadgetItems(#LISTICONGADGET_PANEL_HYPHENATOR_DIC)>0
          For f=1-1 To CountGadgetItems(#LISTICONGADGET_PANEL_HYPHENATOR_DIC)-1
            t3$+GetGadgetItemText(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,f,0)+#LF$
            t3$+GetGadgetItemText(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,f,1)+#LF$
            t3$+GetGadgetItemText(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,f,2)+#LF$
            t3$+Chr(9)
          Next f
        EndIf             
        CloseWindow(#WINDOW_MAIN)        
        draw_GUI()
        SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,t$)
        SetGadgetText(#EDITORGADGET_PANEL_HYPHENATOR_DIC,t1$)
        SetGadgetText(#STRING_PANEL_HYPHENATOR_WORD,t2$)
        If t3$<>""
          For f=1 To CountString(t3$,Chr(9))
            AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(t3$,f,Chr(9)))
          Next f
        EndIf
        update_number_words_for_hyphenation()
        SetGadgetState(#PANEL_MAIN,current_tab) ; Fixed on 20/DEC/2016
        BindGadgetEvent(#PANEL_BOX_DICTIONARY_VERTICAL_BAR, @update_vertical_scroll_bar_dictionary()) ; Dictionary
        BindGadgetEvent(#PANEL_BOX_SYNONYMS_VERTICAL_BAR, @update_vertical_scroll_bar_thesaurus())    ; Thesaurs
        BindGadgetEvent(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR, @update_vertical_scroll_bar_hyphenation()) ; Hyphenation List
        BindGadgetEvent(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR, @update_vertical_scroll_bar_autocorrect_list()) ; Autocorrect List        
        update_filename_dictionary()
        update_filename_synonyms()
        update_filename_hyphenation()
        update_filename_autocorrect_list()
      EndIf

      ; Change the paging of the vertical scroll bars
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)
      SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)
      SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)      
      SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_PageLength,number_of_lines)        

      ; Reset dictionary
      selected_dictionary_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
      update_vertical_scroll_bar_dictionary()

      ; Reset thesaurus
      selected_thesaurus_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
      update_vertical_scroll_bar_thesaurus()
      
      ; Reset hyphenation
      selected_hyphenation_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
      update_vertical_scroll_bar_hyphenation()
      
      ; Reset autocorrect
      selected_autocorrect_word=-1 ; Reset the current word selected
      SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
      update_vertical_scroll_bar_autocorrect_list()                
      
      ; Place back the first item on top of the ListIconGadget
      SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,top_of_listicongadget_dictionary)
      SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,top_of_listicongadget_thesaurus)
      SetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,top_of_listicongadget_autocorrect)
      update_vertical_scroll_bar_dictionary()
      update_vertical_scroll_bar_thesaurus()
      update_vertical_scroll_bar_autocorrect_list()   
      
      ; Reset the .AFF Aid current combobox position if changed the country
      If aff_aid_language$<>old_aff_aid_language$ : aff_aid_language_current_combobox_entry=0 : EndIf
      
      
    EndIf
      
        
    ; Close the Preferences window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    If IsWindow(#WINDOW_PREFERENCES) : CloseWindow(#WINDOW_PREFERENCES) : EndIf ; Close it only, if the changing of window resolution didn't do it
    
  EndProcedure

; IDE Options = PureBasic 5.62 (Windows - x64)
; CursorPosition = 514
; FirstLine = 534
; Folding = -
; EnableXP
; CompileSourceDirectory
; EnableUnicode