﻿  ; Load the words from a .DIC file (Dictionary)
  ;
  ; V1.0 - 5/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 5/JUN/2013
  ;        o Coded the function
  ; V1.2 - 7/JUN/2013
  ;        o It now loads the .AFF file and checks if it is in UTF-8,
  ;          showing a warning if not.
  ; V1.3 - 8/JUN/2013
  ;        <ESC> now aborts loading
  ; V1.4 - 20/JUN/2013
  ;        o A HUGE speed gain thanks to Ashley from the PureBasic Forum
  ; V1.5 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.6 - 26/JUN/2013
  ;        o Changed the non UTF-8 warning to a better one
  ;        o Changed the WindowTimer code To make it work better (faster) in Linux.
  ; V1.7 - 8/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.8 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.9 - 7/FEB/2014
  ;        o Code improved
  ; V2.0 - 27/FEB/2014
  ;        o Removed the file buffer
  ;        o Array used instead of listicongadget
  ;        o Fix: The progress bar now works in Linux
  ; V2.1 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1    
  ; V2.2 - 21/MAR/2014  
  ;        o Resets progress bar when task completes  
  ; V2.3 - 16/NOV/2014    
  ;        o Simplified and optimised the code thanks to Bernd from the PureBasic forum
  ; V2.4 - 21/NOV/2014      
  ;        o Converts .DIC words separated from tags by a space (ex: Galician) asking first.
  ; V2.5 - 3/DEC/2014        
  ;        o Fixed a bug of adding multiple tabs while opening a .DIC with spaces instead of tabs
  ; V2.6 - 22/APR/2015
  ;        o In the requesters complaining about "UTF-8" it now says "UTF-8 without BOM"
  ; V2.7 - 30/OCT/2015
  ;        o Replaced the "UTF-8 W/O BOM" with "UTF-8 WITHOUT BOM" to make it more understandable.
  ; V2.8 - 23/MAR/2016
  ;        o Used the flag #PB_ByteLength in PeekS to avoid trash at the end of the .AFF file when it is loaded
  ; V2.9 - 29/AUG/2016
  ;        o Simplified the .AFF loading
  ; V3.0 - 22/DEC/2016
  ;        o Started implementing the multiselection.
  ; V3.1 - 7/JUN/2017
  ;        o undo/redo support  
  ; V3.2 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry 
  ; V3.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V3.4 - 28/MAR/2018
  ;        o Convert .AFF + .DIC (hyphenation) to #LF$
  ;
  Procedure load_dictionary(force_file$)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Dictionary... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    
    ; Confirm if to load a new Dictionary if the file in memory has been changed
    If dictionary_changed=#True
      If MessageRequester("Warning!","The Dictionary has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf    
    
    
    If force_file$=""
      file$=OpenFileRequester("Open Dictionary", "", "Dictionary (*.dic)|*.dic", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
        ProcedureReturn #False
      EndIf
    Else
      file$=force_file$
    EndIf      
    
    
    ; Check if .AFF file exists
    If FileSize(Left(file$,Len(file$)-3)+"aff") <= 0
      MessageRequester("Error", ".AFF file not found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
    
    
    ; Check if .DIC file exists
    If FileSize(file$) <= 0
      MessageRequester("Error", ".DIC file not found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
        
    
    ; Check If .AFF is in UTF-8 format (we presume that the .DIC is in the same format as the .AFF)
    ReadFile(1,Left(file$,Len(file$)-3)+"aff")
    t=#False
    Repeat
      text$=UCase(ReadString(1,#PB_UTF8))
      If Left(text$,3)="SET" : t=#True :EndIf
    Until Eof(1) Or t=#True
    CloseFile(1)
    If Mid(text$,5)<>"UTF-8"
      If MessageRequester("Warning!","Dictionary is not in "+Chr(34)+"UTF-8 WITHOUT BOM"+Chr(34)+" format:"+#LF$+text$+#LF$+#LF$+"It may lead to corrupt results."+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        ProcedureReturn #False
      EndIf
    EndIf    
    
    
    dictionary_disable_undo_redo() ; 7/JUN/2017
    
    
    selected_dictionary_word=-1 ; Reset the current word selected
    dictionary_changed=#False ; Dictionary set to unchanged
    ; Load .AFF file
    SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,"") ; Clear the .AFF text gadget   
    FUCKTIME = ElapsedMilliseconds()    
    ReadFile(1,Left(file$,Len(file$)-3)+"aff")
      buffer$=ReadString(1,#PB_UTF8|#PB_File_IgnoreEOL)    
    CloseFile(1)
    buffer$=convert_to_linux(buffer$)
    SetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF,buffer$)
             
    
    ; Read .DIC file
    If number_of_words>0 : ClearGadgetItems(#PANEL_BOX_DICTIONARY) : EndIf ; To remove the current words from the gadgetlist    
    convert_space_to_tab=-1 ;Convert space to tab (Galician dictionary) -1 = Didn't ask | 0 = Asked and NO | 1 = Asked and YES
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    ReadFile(1,File$)  
      number_of_words=0
      Dim words$(0)
      Dim words_selected(0)
      number_of_words_total_selected=0
      text$=ReadString(1,#PB_UTF8) ;Just to read the header of the .DIC which contains the number of words
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,Lof(1))
      Repeat
        word$=ReadString(1,#PB_UTF8)
        ; Ask here to convert space to tab *START*
        If FindString(word$,#TAB$)<>0 : convert_space_to_tab=0 : EndIf ; 3/DEC/2014 - Fixed a bug of adding multiple tabs while opening a .DIC with spaces instead of tabs
        If FindString(word$,#TAB$)=0 And (FindString(word$,"[")<>0 Or FindString(word$,"]")<>0 Or FindString(word$,",")<>0 Or FindString(word$,"=")<>0 Or FindString(word$,":")<>0) And convert_space_to_tab=-1
          If MessageRequester("Warning!","This .DIC seems to separate Words from Tags using a space."+#LF$+#LF$+"Use first space as a separator?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            convert_space_to_tab=1 ; YES
          Else
            convert_space_to_tab=0 ; NO
          EndIf      
        EndIf
        ; Ask here to convert space to tab *END*
        If convert_space_to_tab=1
          ReplaceString(word$," ",#LF$,#PB_String_InPlace,1,1)
        Else
          ReplaceString(word$,#TAB$,#LF$,#PB_String_InPlace,1,1)
        EndIf
        ReDim words$(number_of_words)
        ReDim words_selected(number_of_words)
        words$(number_of_words)=word$
        number_of_words+1
;        AddGadgetItem(#PANEL_BOX_DICTIONARY,-1,word$)
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          update_number_words()
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,Loc(1),#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)           
            ProcedureReturn #False
         EndIf
        EndIf

      Until Eof(1)
    CloseFile(1)  
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    update_number_words()
    RemoveWindowTimer(#WINDOW_MAIN,800)
    dictionary_words_filename$=file$
    update_filename_dictionary()
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 21/MAR/2014
      
    ; New code to handle the vertical scroll bar
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
    update_vertical_scroll_bar_dictionary()
    
    ; 31/OCT/2017 - Support for "Recent Files" menu entry
    create_load_update_recent_files(file$)
    update_recent_files_menu()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
  EndProcedure
  
  
  
  
  ; Save/Save As the words to a .DIC file (Dictionary) + .AFF file
  ; V1.0 - 5/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 16/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 20/JUN/2013
  ;        o HUGE speed gain saving the .AFF file and removed the first phase
  ;          that would first convert the listbox to an array and then save
  ;          the array.
  ; V1.3 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.4 - 26/JUN/2013
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.5 - 11/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.6 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.7 - 27/FEB/2014
  ;        o Removed the file buffer
  ;        o It now works with V3+ words$() 
  ; V1.8 - 28/FEB/2014
  ;        o Changed the WindowTimer code To make it work better (faster) in Linux.
  ;        o Fix: The progress bar now works in Linux
  ; V1.9 - 27/MAR/2014  
  ;        o Resets progress bar when task completes   
  ; V2.0 - 3/DEC/2014
  ;        o Simplified and optimised the save dictionary code thanks to Bernd from the PureBasic forum.
  ; V2.1 - 11/SEP/2016
  ;        o Now it saves Dictionary + Thesaurus + Autocorrect with #LF$ instead of #CRLF for Linux compatibility
  ; V2.2 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry
  ; V2.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.
  ;
  Procedure save_saveas_dictionary(save, saveas)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Dictionary... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
    If t$=""
      MessageRequester("Warning!","The .AFF file is empty.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf    
    
    If save=1 : file$=dictionary_words_filename$ : EndIf
    
    If save=1 And dictionary_words_filename$="" : saveas=1 : EndIf
    
    If saveas=1
      file$=SaveFileRequester("Save Dictionary", "", "Dictionary (*.dic)|*.dic", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf
      If LCase(Right(file$,4))<>".dic" : file$+".dic": EndIf
    EndIf
    

    If FileSize(file$)>0 And saveas=1
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    
    DeleteFile(file$) ; Delete .DIC file
    DeleteFile(Left(file$,Len(file$)-4)+".aff") ; Delete the .AFF file
    
    
    ; Save the .AFF file, using UTF-8 encoding
    t$=convert_to_linux(GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF))
    OpenFile(1,Left(file$,Len(file$)-4)+".aff")
      WriteString(1,t$,#PB_UTF8)
    CloseFile(1)
              

    ; Saving the .DIC file
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,File$)
      WriteString(1,Str(number_of_words)+#LF$,#PB_UTF8) ; Write the # of words  
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
      time=ElapsedMilliseconds()
      FUCKTIME=ElapsedMilliseconds()
      For f=0 To number_of_words-1
        text$=words$(f)
        ReplaceString(text$,#LF$,Chr(9),#PB_String_InPlace,1)
        WriteString(1,text$+#LF$,#PB_UTF8)
                      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        

      Next f
      Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      CloseFile(1)
      
      ; Dictionary set to unchanged
      dictionary_changed=#False
      update_number_words()
      
      ; 31/OCT/2017 - Support for "Recent Files" menu entry
      create_load_update_recent_files(file$)
      update_recent_files_menu()      
      
      dictionary_words_filename$=file$
      update_filename_dictionary()
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
    
  EndProcedure
  
  
  
  
  ; Updates the number of words in the Dictionary
  ; Useful in open, add, delete
  ; V1.0 - 5/JUN/2013
  ; V1.1 - 21/JUN/2013
  ;        o Improved the code thanks to a tip from Bernd (PureBasic forum)
  ; V1.2 - 16/JUL/2013
  ;        o Now places a "*" if the file has been changed  
  ; V1.3 - 7/FEB/2014
  ;        o Code improved
  ; V1.4 - 13/MAR/2014
  ;        o Like suggested by Filiep, it now shows: "Word: [211/588557]"
  ; V1.5 - 25/OCT/2017
  ;        o Formats the total number of items to be better seen.
  ; V1.6 - 20/JUL/2018
  ;        o Now the changes allow a colour asterisk in red
  ; V1.7 - 23/JUL/2018
  ;        o Fix: Asterisks in red no longer flash
  ; V1.8 - 27/JUL/2018
  ;        28/JUL/2018
  ;        o Fix: Reduced asterisks vanishing if scrolling holding down the arrow keys  
  ; V1.9 - 19/SEP/2018
  ;        o Floppy icon on tabs (blue/red)
  ;
  Procedure update_number_words()
    
    ; Black/Red floppy
    If dictionary_changed=#False And GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK)<>""
      SetGadgetItemImage(#PANEL_MAIN,0,ImageID(71))
    EndIf
    If dictionary_changed=#True And GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK)<>"*"
      SetGadgetItemImage(#PANEL_MAIN,0,ImageID(72))
    EndIf    
    
    t$=""
    If dictionary_changed=#True : t$="*" : EndIf        
    If selected_dictionary_word=-1
      t1$="# Words:"+FormatNumber(number_of_words,0,".",number_separator_chr$)
    Else
      t1$="Word: ["+FormatNumber(selected_dictionary_word+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_words,0,".",number_separator_chr$)+"]"
    EndIf
    
    If t1$<>GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS)
      gadget=#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS
      gadget2=#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK
      w=GadgetWidth(gadget)
      SetGadgetText(gadget,t1$)
      GadgetFit(gadget)
      If GadgetX(gadget2)<GadgetX(gadget)+GadgetWidth(gadget) Or GadgetX(gadget2)>=GadgetX(gadget)+GadgetWidth(gadget)+4
        ResizeGadget(gadget2,GadgetX(gadget)+GadgetWidth(gadget),GadgetY(gadget),16,16)                        
        GadgetRefresh(gadget2)  
      EndIf
    EndIf   
    
    If GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK)<>t$
      SetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_ASTERISK,t$)
    EndIf
    
  EndProcedure  
  
  
  
  
  ; Updates the filename of the words in the Dictionary
  ; Used in open and save as
  ;
  ; V1.0 - 11/JUN/2013
  ; V1.1 - 21/JUN/2013
  ;        o Improved the code thanks to a tip from Bernd (PureBasic forum)
  ; V1.2 - 11/FEB/2016
  ;        o Simplified the code  
  ;
  Procedure update_filename_dictionary()
    a$=""
    If Len(dictionary_words_filename$)>0
      t=CountString(dictionary_words_filename$,"/")
      If t>0 : a$=StringField(dictionary_words_filename$,t+1,"/") : EndIf
      t=CountString(dictionary_words_filename$,"\")
      If t>0 : a$=StringField(dictionary_words_filename$,t+1,"\") : EndIf
    EndIf      
    If a$="" : a$="n/a" : EndIf
    SetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_FILENAME,"Filename:"+a$)
  EndProcedure  
  
  
  
  
  ; Sort the Dictionary
  ; V1.0 - 12/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 24/MAY/2013
  ;        o Coded the complete procedure
  ; V1.2 - 20/JUN/2013
  ;        o A HUGE speed gain thanks to Ashley from the PureBasic Forum
  ; V1.3 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.4 - 26/JUN/2013
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.5 - 12/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.6 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.7 - 27/FEB/2014
  ;        o It now works with V3+ words$()
  ; V1.8 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1  
  ; V1.9 - 21/MAR/2014  
  ;        o Resets progress bar when task completes  
  ; V2.0 - 22/DEC/2016
  ;        25/DEC/2016
  ;        o Started implementing the multiselection.
  ; V2.1 - 7/JUN/2017
  ;        o undo/redo support
  ; V2.2 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.3 - 3/APR/2018
  ;        o Support for hyphenation
  ;
  Procedure sort_dictionary()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting Dictionary...")
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn 0
    EndIf
    
    dictionary_store_undo() ; 7/JUN/2017
    
    ; Show the progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,1) 
    
    ; Dictionary set to changed
    selected_dictionary_word=-1 ; Reset the current word selected
    dictionary_changed=#True
    update_number_words()
        
    FUCKTIME=ElapsedMilliseconds()
    ; Sort the array and delete the list 
    SortArray(words$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
    ClearGadgetItems(#PANEL_BOX_DICTIONARY)       
    SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,0)
    update_vertical_scroll_bar_dictionary()
    
    ; Fill progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,1,#PB_StatusBar_Raised,0,1) 
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,1) ; Reset progress bar - 21/MAR/2014
    
    ; No selected items
    update_number_words_selected_none()
    
    
    ; Hyphenation support - 3/APR/2018
    selected_hyphenation_word=-1 ; Reset the current word selected
    update_number_hyphenations()
    ClearGadgetItems(#LISTICONGADGET_PANEL_HYPHENATOR)       
    SetGadgetState(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,0)
    update_vertical_scroll_bar_hyphenation()    
    
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
            
    
  EndProcedure  
  
  
  
  
  ; Delete words in the Dictionary
  ;
  ; V1.0 - 12/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 14/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ; V1.3 - 28/FEB/2014
  ;        o It now works with V3+ words$() 
  ; V1.4 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1    
  ;          (for DELETE, only if the number of words become 0)
  ; V1.5 - 27/MAR/2014
  ;        o It now allows deleting words not currently visible in the listicongadget
  ; V1.6 - 1/AUG/2014
  ;        o It now shows the words to be deleted inside quotation marks
  ; V1.7 - 23/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.8 - 7/JUN/2017
  ;        o undo/redo support
  ; V1.9 - 6/NOV/2017
  ;        o Now recognises being called from advanced find  
  ; V2.0 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.1 - 3/APR/2018
  ;        o Support for hyphenation
  ;
  Procedure delete_word(is_called_from_advance_find)
    
    If number_of_words=0 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    t=selected_dictionary_word
    If number_of_words>0 And t=-1 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    If number_of_words>0 And t<>-1      
      text$=words$(t)
      text$=ReplaceString(text$,#LF$," ")
      If MessageRequester("Warning!","Really delete word?"+#LF$+Chr(34)+text$+Chr(34),#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
;        RemoveGadgetItem(#PANEL_BOX_DICTIONARY,GetGadgetState(#PANEL_BOX_DICTIONARY))        
;        t1=GetGadgetState(#PANEL_BOX_DICTIONARY)+GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)        
        flag=#False 
        
        dictionary_store_undo() ; 7/JUN/2017
        
        ; Only one word in the dictionary
        If number_of_words=1 And flag=#False
          ReDim words$(0)
          ReDim words_selected(0)
          words_selected(0)=#False
          flag=#True
        EndIf
        
        ; The word to delete is the last
        If number_of_words=t+1 And flag=#False
          ReDim words$(ArraySize(words$())-1)
          ReDim words_selected(ArraySize(words_selected())-1)
          flag=#True
        EndIf
        
        ; The word to delete is inside the list, not the first nor the last item
        If flag=#False
          For f=t+1 To number_of_words-1
            words$(f-1)=words$(f)
            words_selected(f-1)=words_selected(f)
          Next f
          ReDim words$(ArraySize(words$())-1)          
          ReDim words_selected(ArraySize(words_selected())-1)   
          flag=#True
        EndIf
        
        number_of_words-1
        update_number_words_selected_count()
        SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        selected_dictionary_word=-1 ; Reset the current word selected
        
        dictionary_changed=#True
        update_number_words()
        update_vertical_scroll_bar_dictionary()
        
        
        ; Hyphenation support - 3/APR/2018
        SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        selected_hyphenation_word=-1 ; Reset the current word selected      
        update_number_hyphenations()
        update_vertical_scroll_bar_hyphenation()
        
        ProcedureReturn #True
        
      EndIf
    EndIf
    
    ProcedureReturn #False
    
  EndProcedure    
  
  
  
  
  ; Erase words in the Dictionary
  ; V1.0 - 13/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 13/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 14/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ;        o After a purge, the filename is reset.
  ; V1.3 - 17/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed
  ;        o Checks if there have been changes, and ask before purging
  ; V1.4 - 27/FEB/2014
  ;        o It now works with V3+ words$()
  ; V1.5 - 13/MAR/2014
  ;        o Reset the current word selected: selected_dictionary_word=-1    
  ; V1.6 - 21/MAR/2014  
  ;        o Resets progress bar when task completes   
  ; V1.7 - 23/MAR/2014  
  ;        o Fix: Purge Dictionary didn't reset the vertical scroll bar
  ; V1.8 - 22/DEC/2016
  ;        24/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.9 - 7/JUN/2017
  ;        o undo/redo support
  ; V2.0 - 1/DEC/2017
  ;        o Message Requesters now have a sign.
  ;
  Procedure erase_dictionary()
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf
    
    ; Confirm if to erase the Dictionary if the file in memory has been changed
    If dictionary_changed=#True
      If MessageRequester("Warning!","The Dictionary has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf     
    
    If number_of_words>0
      If MessageRequester("Warning!","Really delete all words?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        
        dictionary_store_undo() ; 7/JUN/2017
        
        ; Reset total number of selected words
        update_number_words_selected_none()        
        
        FUCKTIME=ElapsedMilliseconds()
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Erasing...")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        number_of_words=0
        ClearGadgetItems(#PANEL_BOX_DICTIONARY)
        Dim words$(0)
        Dim words_selected(0)
        number_of_words_total_selected=0
        update_number_words()
        dictionary_words_filename$=""
        update_filename_dictionary()
        
        selected_dictionary_word=-1 ; Reset the current word selected
        dictionary_changed=#False ; Dictionary set to unchanged
        update_number_words()
        update_vertical_scroll_bar_dictionary()
        Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
        
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,100,#PB_StatusBar_Raised,0,100)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,100) ; Reset progress bar - 21/MAR/2014
                
        
      EndIf
    EndIf
        
  EndProcedure    
  
 
  
  
  ; ADD/EDIT WORD
  ; If add=1 and edit=0 then it ADDS
  ; If add=0 and edit=1 then it EDITS
  ; V1.0 - 13/JUN/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 15/JUN/2013
  ;        o Coded the procedure
  ; V1.2 - 16/JUL/2013
  ;        o It now uses the variable dictionary_changed to tell it has been changed  
  ; V1.3 - 7/AUG/2013
  ;        8/AUG/2013
  ;        o Initial support for derived suggestions
  ; V1.4 - 10/AUG/2013
  ;        o Added to the list box the field "Type"
  ; V1.5 - 11/AUG/2013
  ;        o Started coding the PFX and SFX
  ; V1.6 - 14/AUG/2013
  ;        o Added the Suffixes code
  ; V1.7 - 15/AUG/2013  
  ;        o The derived words listbox now has a column with the code of each word
  ; V1.8 - 16/AUG/2013
  ;        o The search for Suffixes and Prefixes now give priority to "N" instead of "Y"
  ; v1.9 - 17/AUG/2013
  ;        o Coded a display button to find the affix/prefix in the .AFF
  ;        o Added a position column
  ; V2.0 - 23/AUG/2013
  ;        o Added a WindowTimer to avoid the window to stop responding
  ; V2.1 - 24/AUG/2013
  ;        o Optimised the code a little.
  ;        o (fix) In case letters are typed during a refresh, to refresh again
  ; V2.2 - 30/AUG/2013
  ;        o Coded the DISPLAY function, thanks to Ash and to a forum member
  ; V2.3 - 2/SEP/2013
  ;        o Coded the double-clicking in derived words to move to the .AFF position
  ; V2.4 - 31/OCT/2013
  ;        o First I allowed only one SFX or PFX by code
  ;        o Now it allows both - en_GB "O" code is used by SFX and PFX
  ; V2.5 - 11/FEB/2014
  ;        o Ubuntu/Linux users can close the window in its gadget
  ; V2.6 - 13/FEB/2014
  ;        o Check if <ESC> was pressed
  ; V2.7 - 27/FEB/2014
  ;        o It now works with V3+ words$()
  ; V2.8 - 21/MAR/2014 
  ;        o Improved: While editing a word, the cursor focus is now in the "Word:" textbox.  
  ; V2.9 - 24/MAR/2014 
  ;        o Improved the code to work better with V3+ (selected word not visible in listicongadget)
  ; V3.0 - 18/APR/2014
  ;        o Added the shortcut <ENTER>
  ;        o If the word/flag has a space at the beginning or end, the text box becomes red
  ; V3.1 - 20/APR/2014  
  ;        o It now searches for two spaces in the word, making the text box become red.
  ; V3.2 - 8/JUL/2014
  ;        10/JUL/2014
  ;        o Replaced "Flags" with "POStag".
  ; V3.3 - 11/JUL/2014  
  ;        o Replaced "POStags" with "Tags" as suggested by Filiep.
  ;        o Checks if word ends with "/" or has more than one "/" (invalid).
  ;        o Moved part of the code to extract the derivates, in order to be used by the new menu.
  ; V3.4 - 12/JUL/2014    
  ;        o Fix Regression from build 56: sometimes while writing too fast, the listicongadget wasn't refreshed.
  ;        o Added and coded the "Rule" column
  ; V3.5 - 13/JUL/2014
  ;        o Fix: It no longer shows "Primary" for invalid words (blank OR starting with a "/")
  ; V3.6 - 14/AUG/2014
  ;        o Increased the width of the editor/listicon gadgets, plus the "rules" field.
  ; V3.7 - 5/NOV/2014
  ;        o Now it only sets the "changed" flag, if the word/tags are really changed.
  ; V3.8 - 15/NOV/2014
  ;        o It now accepts: \/ are escaped "/"  
  ; V3.9 - 17/JAN/2015
  ;        o Ultrafast searching for rules positions  
  ; V4.0 - 29/OCT/2015
  ;        o Increased the width from 640 to 800 to show more content.
  ; V4.1 - 2/NOV/2015
  ;        o Replaced the listicongadget field "Position" with "Code Position".
  ; V4.2 - 9/NOV/2015
  ;        o Simplified the code a bit
  ; V4.3 - 20/JAN/2016
  ;        o "Display" button now disabled if nothing is selected in the ListIconGadget
  ; V4.4 - 5/OCT/2016
  ;        o Pop-up menu in words editor to copy the selected line word into the clipboard
  ; V4.5 - 15/DEC/2016
  ;        o Dictionary editor now has a larger window
  ; V4.6 - 23/DEC/2016
  ;        o Started implementing the multiselection.
  ; V4.7 - 6/JAN/2017
  ;        o Now makes uses of ".AFF Aid"
  ; V4.8 - 11/MAR/2017
  ;        o Changed the buttons height to 25.   
  ; V4.9 - 4/APR/2017
  ;        o Support via coloured symbol for NOSUGGEST in words editor 
  ; V5.0 - 7/JUN/2017
  ;        o undo/redo support
  ; V5.1 - 23/JUL/2017
  ;        o F1 (help) now works in the edit/new items windows    
  ; V5.2 - 21/NOV/2017
  ;        o Added "Missing Codes:" to the Dictionary editor.
  ; V5.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V5.4 - 7/FEB/2018
  ;        o ComboBox with higher height (+6 pixels) for better Ubuntu look.
  ; V5.5 - 14/FEB/2018
  ;        o Now enlarges according to window size definition
  ; V5.6 - 3/APR/2018
  ;        o Support for hyphenation
  ; V5.7 - 13/APR/2018
  ;        o Support for compressed AF
  ; V5.8 - 18/MAY/2018
  ;        o Taboo's warning sign is now an image and is stored in a long instead of string for speed up.
  ; V5.9 - 23/MAY/2018
  ;        o Combobox of ".AFF Aid" wider for bigger windows.
  ; V6.0 - 14/JUN/2018
  ;        o Image for NEEDAFFIX and FORBIDDENWORD
  ; V6.1 - 30/JUN/2018
  ;        o Fix: NEEDAFFIX and FORBIDDENWORD variables reset at start of ADD/EDIT
  ; V6.2 - 5/JUL/2018
  ;        o Dictionary editor: Code (flag) is now an emoji: 🏁
  ;        o Taboo image now has a tip
  ; V6.3 - 7/JUL/2018
  ;        o Replaced the word "Tags" with "Postags"
  ; V6.4 - 8/JUL/2018
  ;        o Missing flags is now written in red.
  ;        o Colours for "Suffixes" (#GreenDark) and "Prefixes" (#Blue)
  ; V6.5 - 9/JUL/2018
  ;        o Better check if word is valid before exiting.
  ; V6.6 - 7/SEP/2018
  ;        o Pop-up menus now use the flag: #PB_Menu_ModernLook (Windows only)
  ;
  Procedure add_edit_word(add,edit)

    If number_of_words=0 And edit=#True
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    t=selected_dictionary_word
    If t=-1 And edit=#True
      MessageRequester("Warning!","No word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
     
    t$="Add a word" ;add=#True
    If edit=#True : t$="Edit a word" : EndIf
    If OpenWindow(#WINDOW_ADD_EDIT,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_ADD_EDIT,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)
    
    ; Word field
    TextGadget(#TEXT_WINDOW_ADD_EDIT_WORD,0+6,0+6+2+1,80,16,"Word:")
    t=selected_dictionary_word
    t$=""
    If t<>-1 And edit=#True
      t$=StringField(words$(t),1,#LF$)
    EndIf
    StringGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD,3+0+6+100-15-15-20,0+6,320-10-20-40+20,20+3,t$)
    SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
    old_word$=t$ ; 5/NOV/2014
    
    ; NOSUGGEST fields
    TextGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO,GadgetX(#TEXTBOX_WINDOW_ADD_EDIT_WORD)+GadgetWidth(#TEXTBOX_WINDOW_ADD_EDIT_WORD)+5+8+3,GadgetY(#TEXTBOX_WINDOW_ADD_EDIT_WORD)+2,40,16,"Taboo:")    
    ImageGadget(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,GadgetX(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO)+GadgetWidth(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO)+5,GadgetY(#TEXTBOX_WINDOW_ADD_EDIT_WORD_TABOO)-3-1,24,20,ImageID(49))
    GadgetToolTip(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,"NOSUGGEST Flag")
    
    ; POSTAGS field
    TextGadget(#TEXT_WINDOW_ADD_EDIT_FLAG,0+6,0+6+2+1+20+5+2,80,16,"Postags:")
    t$=""
    If t<>-1 And edit=#True
      t1=FindString(words$(t),#LF$)
      If t1>0 : t$=StringField(words$(t),2,#LF$) : EndIf
    EndIf    
    StringGadget(#EDITOR_WINDOW_ADD_EDIT_TAG,3+0+6+100-15-15-20,0+6+2+1+20+5,600-20-40-20-10-20-40+20-100,20+3,t$)      
    old_tags$=t$ ; 5/NOV/2014
    
    ; Add the OKAY and CANCEL buttons
    ButtonGadget(#BUTTON_ADD_EDIT_OKAY,5-20-20+0+6+100-15,30+30+3,90,25,"Okay")
    ButtonGadget(#BUTTON_ADD_EDIT_CANCEL,5-20-20+0+6+100-15+100,30+30+3,90,25,"Cancel")
    
    ; Derived gadget listbox
    ListIconGadget(#PANEL_BOX_DERIVED_WORDS,10+40+5,200-40-50-20-20+60-20+20+20-40-5,get_extra_width(100)+3+(1024-800)+(800-640)+10+320+100-30-50+5+155+20+20,60+100+20+5+40-5+get_extra_height(20),"Derived words",10-10+320+100-30-50+5-5-100-20-10-10-10-10-10-10,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,1,"Type",150-20-20-10-10-10-5)    
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,2,"🏁",150-20-20-10-10-10-5-5-5-2-5+10)        
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,3,"🏁 Position",10+15+150-20-20-10-10-10-5-2+5)        
    AddGadgetColumn(#PANEL_BOX_DERIVED_WORDS,4,"Rule",get_extra_width(15)+20+15+5+15+150-20-20-10+10+10+10+10+5+10+100-5-5-5-10+20+40+40)
    ButtonGadget(#PANEL_BOX_DERIVED_WORDS_DISPLAY,10+40+5+10-5-3,200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2-2+60+get_extra_height(20),90,25,"Display")       
    TextGadget(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,10+100+20+30+5,200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2+60+get_extra_height(20),400,16,"# Derived Words:0")
    
    ; AFF Text + Editor Gadget
    TextGadget(#TEXT_WINDOW_ADD_EDIT_AFF,0+6,get_extra_height(20)+2+200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2+20+5+2+2+60,60-15,16,".AFF:")
    EditorGadget(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD,0+6+2+1+20+5+10+10,get_extra_height(20)+2+200-40-50-20-20+60-20+20+20-40-5+100+20+5+40-5+10-2-2+20+5+2+2+60,get_extra_width(100)+(1024-800)+(800-640)+15+200+20+30+30+10+3+100+100+100-50,get_extra_height(80)+100+20+60,#PB_Editor_ReadOnly)    
    SetGadgetText(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD,derived_aff_nospaces$)
    
    ; AFF Aid
    x=GadgetX(#EDITOR_WINDOW_ADD_EDIT_TAG)+GadgetWidth(#EDITOR_WINDOW_ADD_EDIT_TAG)+10+20
    y=5    
    t$="n/a"
    For f=0 To ArraySize(aff_aid_language_array$())
      If StringField(aff_aid_language_array$(f),3,#LF$)<>"" : image=Val(StringField(aff_aid_language_array$(f),2,#LF$)) : EndIf
      If aff_aid_language$=StringField(aff_aid_language_array$(f),3,#LF$)
        current_aff_aid_language=f
        t$=StringField(aff_aid_language_array$(f),1,#LF$)
        Break
      EndIf  
    Next f    
    TextGadget(#TEXT_WINDOW_ADD_EDIT_AFF_AID,x,y,100+200+150+100-10-3,16,".AFF Aid "+"("+t$+"):")
    y+16+2
    ComboBoxGadget(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,x,y,get_extra_width(100)+100+200+150+100-10-3,20+2+6,#PB_ComboBox_Image)
    populate_combobox_aff_aid(image,current_aff_aid_language)
    SetGadgetState(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,aff_aid_language_current_combobox_entry)
        
    ; Missing Codes
    TextGadget(#TEXT_MISSING_CODES,GadgetX(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID),1+1+1+GadgetY(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+GadgetHeight(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+5+3,100-40+10+10+10+10-20-10-6,16,"Missing 🏁 ")
    GadgetFit(#TEXT_MISSING_CODES)
    SetGadgetColor(#TEXT_MISSING_CODES,#PB_Gadget_FrontColor,#Red)
    TextGadget(#TEXT_MISSING_CODES_2,-6+GadgetX(#TEXT_MISSING_CODES)+GadgetWidth(#TEXT_MISSING_CODES)+5,1+1+1+GadgetY(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+GadgetHeight(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)+5+3,3,16,":")
    StringGadget(#STRING_MISSING_CODES,4+GadgetX(#TEXT_MISSING_CODES_2)+GadgetWidth(#TEXT_MISSING_CODES_2)+5,-1+1+GadgetY(#TEXT_MISSING_CODES_2)-2,100+150,20+3,"",#PB_String_ReadOnly)
    SetGadgetColor(#STRING_MISSING_CODES,#PB_Gadget_BackColor,-1)
    
    ; NEEDAFFIX + FORBIDDENWORD
    ImageGadget(#IMAGE_NEEDAFFIX,GadgetX(#STRING_MISSING_CODES)+GadgetWidth(#STRING_MISSING_CODES)+5,GadgetY(#STRING_MISSING_CODES),53,23,ImageID(56))
    GadgetToolTip(#IMAGE_NEEDAFFIX,"NEEDAFFIX Flag")
    ImageGadget(#IMAGE_FORBIDDENWORD,GadgetX(#IMAGE_NEEDAFFIX)+GadgetWidth(#IMAGE_NEEDAFFIX)+5,GadgetY(#IMAGE_NEEDAFFIX),53,23,ImageID(57))
    GadgetToolTip(#IMAGE_FORBIDDENWORD,"FORBIDDENWORD Flag")
    needaffixword_code=#False
    forbiddenword_code=#False
    
    ; Wait for the user to press the OKAY or CANCEL button
    flag=#False ;to be used if text box with words is changed. #False=unchanged, #True=changed
    time=ElapsedMilliseconds()
    
    ;In case we are editing a word, it shows instantely the derived words
    If edit=#True
      flag=#True
      time=time-300-10
    EndIf
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Return,1003)    
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_F1,1004)    

    ; Create Popup menus
    CreatePopupImageMenu(#POPUPMENU_WINDOW_ADD_EDIT,#PB_Menu_ModernLook)
    MenuItem(#POPUPMENU_WINDOW_ADD_EDIT_COPY,"Copy word")    
    
    okay=#False
    cancel=#False
; *** CHECK THE CHRS **** START ****
; t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)    
; For f=1 To Len(t$)
;   Debug Mid(t$,f,1)+","+Asc(Mid(t$,f,1))
; Next f    
; *** CHECK THE CHRS **** END ****    
    Repeat
      
      
      ; Change background of word string gadget if a space is found at the start or end, or two spaces in a row (invalid word)  
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
      If t$<>"" And (((Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0) Or (Right(t$,1)="/" And Right(t$,2)<>"\/") Or (Right(t$,1)="/" And Right(t$,3)<>"\//")))
        If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>#Red: SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor,#Red) : EndIf
      Else
        If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>-1: SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor,-1) : EndIf
      EndIf    
      
      ; Change background of flag string gadget if a space is found at the start or end (invalid word)  
      t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
      If t$<>"" And (Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0)
        If GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor)<>#Red: SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor,#Red) : EndIf
      Else
        If GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor)<>-1: SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_TAG,#PB_Gadget_BackColor,-1) : EndIf
      EndIf             
      
      
      ; Disable/Enable the "Display" button according to selected/unselected items in the ListIconGadget
      disable_button_display=#False
      If GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1 : disable_button_display=#True : EndIf
      DisableGadget(#PANEL_BOX_DERIVED_WORDS_DISPLAY,disable_button_display)
      
      
      ; NOSUGGEST support - 4/APR/2017
      If (GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)<>-1 Or GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)="") And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(49)
        nosuggestword_colour=#Black
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(49))        
      EndIf
      If nosuggestword_colour=#Green And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(50)
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(50))
      EndIf
      If nosuggestword_colour=#Red And GetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST)<>ImageID(51)
        SetGadgetState(#IMAGE_WINDOW_ADD_EDIT_WORD_NOSUGGEST,ImageID(51))
      EndIf
      
      ; NEEDAFFIX support - 14/JUN/2018
      If needaffixword_code=#True And GetGadgetState(#IMAGE_NEEDAFFIX)=ImageID(56) : SetGadgetState(#IMAGE_NEEDAFFIX,ImageID(54)) : EndIf
      If needaffixword_code=#False And GetGadgetState(#IMAGE_NEEDAFFIX)=ImageID(54) : SetGadgetState(#IMAGE_NEEDAFFIX,ImageID(56)) : EndIf
      
      ; FORBIDDENWORD support - 14/JUN/2018
      If forbiddenword_code=#True And GetGadgetState(#IMAGE_FORBIDDENWORD)=ImageID(57) : SetGadgetState(#IMAGE_FORBIDDENWORD,ImageID(55)) : EndIf
      If forbiddenword_code=#False And GetGadgetState(#IMAGE_FORBIDDENWORD)=ImageID(55) : SetGadgetState(#IMAGE_FORBIDDENWORD,ImageID(57)) : EndIf
      
      
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf     
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf        
      
      
      ; Check if <ENTER> was pressed
      If event=#PB_Event_Menu And EventMenu()=1003
        Debug "OKAY was pressed"
        PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #BUTTON_ADD_EDIT_OKAY)
      EndIf      
      
      
      ; Shortcut for F1 (help)
      If event=#PB_Event_Menu And EventMenu()=1004
        html_help()
      EndIf       
      
      
      ; POP-UP MENU
      If EventGadget()=#PANEL_BOX_DERIVED_WORDS And EventType()=#PB_EventType_RightClick
        DisplayPopupMenu(#POPUPMENU_WINDOW_ADD_EDIT, WindowID(#WINDOW_ADD_EDIT))
      EndIf  
      
      Select event
  
      Case #PB_Event_Menu
      
        Select EventMenu()  ; To see which popup menu has been selected

          Case #POPUPMENU_WINDOW_ADD_EDIT_COPY
            
            flag_popup=#False
            If flag_popup=#False And CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)=0
              MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
              flag_popup=#True
            EndIf
            If flag_popup=#False And GetGadgetState(#PANEL_BOX_DERIVED_WORDS)=-1
              MessageRequester("Warning!","No word selected.",#PB_MessageRequester_Warning)
              flag_popup=#True
            EndIf
            If flag_popup=#False
              SetClipboardText(GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,GetGadgetState(#PANEL_BOX_DERIVED_WORDS),0))
              flag_popup=#True
            EndIf
                        
        EndSelect    
      EndSelect        
            
      
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
           ; Change the word
           Case #TEXTBOX_WINDOW_ADD_EDIT_WORD
             If EventType()=#PB_EventType_Change
                flag=#True            
                time-1000 ; To refresh the word in real time (300 would be the exact number, but used 1000 to be 100% sure it will activate the refresh in real time)
             EndIf  
              
           ; Double-click in a line   
           Case #PANEL_BOX_DERIVED_WORDS
             If EventType()=#PB_EventType_LeftDoubleClick
               PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #PANEL_BOX_DERIVED_WORDS_DISPLAY)
             EndIf
             
           ; Select DISPLAY
           Case #PANEL_BOX_DERIVED_WORDS_DISPLAY             
             If CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)<>0 And GetGadgetState(#PANEL_BOX_DERIVED_WORDS)<>-1
               t=Val(GetGadgetItemText(#PANEL_BOX_DERIVED_WORDS,GetGadgetState(#PANEL_BOX_DERIVED_WORDS),3))-1 ;Get the position of the condition in the .AFF file
               Debug "Position"+Str(t)
               move_to_text_position(t)
             EndIf                                   
             
           ; Press OKAY
           Case #BUTTON_ADD_EDIT_OKAY
             okay=#True
             t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)                               
             If (t$="" Or t$=" ") And okay=#True
               MessageRequester("Warning!","No word text.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
               okay=#False
             EndIf                   
;              If ((Right(t$,1)="/" And Right(t$,2)<>"\/") Or (Right(t$,1)="/" And Right(t$,3)<>"\//")) And okay=#True            
;                MessageRequester("Warning!","Invalid word.",#PB_MessageRequester_Warning)
;                SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
;                okay=#False
;              EndIf             
             If (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And okay=#True
               MessageRequester("Warning!","Invalid spaces in word.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
               okay=#False
             EndIf
             t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG) 
             If Len(t$)<>0 And (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And okay=#True
               MessageRequester("Warning!","Tags incorrect (spaces).",#PB_MessageRequester_Warning)
               SetActiveGadget(#EDITOR_WINDOW_ADD_EDIT_TAG)
               okay=#False          
             EndIf
             If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_WORD,#PB_Gadget_BackColor)=#Red And okay=#True
               MessageRequester("Warning!","Invalid word.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
               okay=#False
             EndIf              
             
           ; Press CANCEL  
           Case #BUTTON_ADD_EDIT_CANCEL
             cancel=#True             
             
          EndSelect
       EndSelect                 
            
 
      
      ;Update derived words
      ; Fix: 13/JUL/2014 - It no longer shows "Primary" for invalid words (blank OR starting with a "/")
      If ElapsedMilliseconds()>time+300 And flag=#True
        ClearGadgetItems(#PANEL_BOX_DERIVED_WORDS)
        colour_line_in_affixes_listicongadget=0
        If GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)<>"" And Left(GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD),1)<>"/"
          process_pfx_sfx(1,GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD),#False,#False,#False,"","","",#False,#False)
        EndIf  
        SetGadgetText(#TEXTBOX_WINDOW_NUMBER_DERIVED_WORDS,"# Derived Words:"+Str(CountGadgetItems(#PANEL_BOX_DERIVED_WORDS)))
        flag=#False  
        time=ElapsedMilliseconds()
      EndIf      
      
      
    Until okay=#True Or cancel=#True
    
    ; OKAY button pressed - Get the current combobox position
    If okay=#True
      aff_aid_language_current_combobox_entry=GetGadgetState(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID)
    EndIf
    
    ; OKAY button pressed. It will now add or update a synonym and its meanings
    If okay=#True
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD)
      a$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
      If a$<>"" : t$+#LF$+a$ : EndIf
      
      If old_word$<>GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD) Or old_tags$<>GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
        dictionary_store_undo() ; 7/JUN/2017
      EndIf
      
      If add=#True
        
        ReDim words$(number_of_words)        
        ReDim words_selected(number_of_words)        
        words$(number_of_words)=t$
        number_of_words+1
        SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        update_number_words()
        update_vertical_scroll_bar_dictionary()  
        
        ; Hyphenation support - 3/APR/2018
        SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
        update_number_hyphenations()
        update_vertical_scroll_bar_hyphenation()
  
      EndIf
      
      If edit=#True
        
        t=selected_dictionary_word         
        words$(t)=t$
        update_vertical_scroll_bar_dictionary()
        update_vertical_scroll_bar_hyphenation() ; Hyphenation support - 3/APR/2018
        
      EndIf
      
    EndIf
    
    ; Okay pressed and change the dictionary_changed variable (ONLY IF WORD/TAGS WERE CHANGED - 5/NOV/2014)
    If okay=#True
      If old_word$<>GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_WORD) Or old_tags$<>GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_TAG)
        dictionary_changed=#True
                
        
      EndIf 
      update_number_words()
      update_number_hyphenations() ; Hyphenation support - 3/APR/2018
    EndIf    
    
    update_vertical_scroll_bar_dictionary_current_item()
    update_vertical_scroll_bar_hyphenation_current_item() ; Hyphenation support - 3/APR/2018
    
    ; Close the Add/Edit window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_ADD_EDIT) 
    
  EndProcedure 
  
  
  
  
  ; This procedure now processes the prefixes and suffixes.
  ; Values for variable: type_of_processing_of_pfx_sfx
  ; 1 = add/edit word
  ; 2 = Extract wordlist
  ; 3 = Count wordlist
  ; 4 = Find duplicates
  ; 5 = Statistics
  ;
  ; According to the value of the variable, for example, if =1 it adds directly in the listicongadget of the add/edit words procedure
  ; If =2 it saves directly to HDD
  ; If =3 it just counts
  ;
  ; Notice that the function returns the number of words found, but this is just useful for =2 and =3
  ; in order to add to the status bar
  ; 
  ; The value of WORD$ is the current word from the array, with all the codes
  ;
  ; The CSV is either #false=export as .txt or #true=export as .csv
  ; The COMPOUNDS if #True means to export compound words
  ;
  ; recursivity=0/1 -> In case it is called a rule from inside a rule
  ;
  ; languagetool$: "" or "*" if exporting LanguageTool compounds
  ;
  ; morphologic_part$: Used in show duplicates
  ;
  ; V1.0 - 11/JUL/2014  
  ; V1.1 - 12/JUL/2014    
  ;        o Optimised the code a bit in the part that searches for Y or N as the header of the rules set
  ;          ( "N" is first searched and only if NOT found, it searches for "Y")
  ;        o Get the number of lines per PFX/SFX is now ultra-fast - just 2 lines instead of 10
  ;        o Added and coded the "Rule" column
  ; V1.2 - 13/JUL/2014      
  ;        o Optimised the bottleneck: Now around 7x faster with the new code
  ; V1.3 - 15/JUL/2014  
  ;        o Replaced the rules text " - " with ": " for the add/edit dictionary
  ; V1.4 - 16/JUL/2014
  ;        o Added CSV
  ; V1.5 - 17/JUL/2014
  ;        o In CSV it now has an extra field with the primary word and the separator is now ";" instead of ",".
  ; V1.6 - 31/JUL/2014  
  ;        o Optimised the code slightly
  ; V1.7 - 6/AUG/2014
  ;        o Coded the "Show Duplicates"
  ; V1.8 - 15/NOV/2014
  ;        o It now accepts: \/ are escaped "/"
  ;          It converts "\/" to ";" and then tries to find a "/" and then it converts ";" to "/"
  ; V1.9 - 8/DEC/2014
  ;        o It now combines PREFIXES against PRIMARY+SUFFIXES
  ; V2.0 - 10/DEC/2014
  ;        o Tried to optimise slightly by only getting the variable value once of Mid(word_codes$,f,1)
  ; V2.1 - 15/DEC/2014
  ;        o Coded statistics
  ; V2.2 - 17/JAN/2015
  ;        o Ultrafast searching for rules positions
  ; V2.3 - 5/MAR/2015
  ;        o Coded "Extract Compounds .txt".
  ; V2.4 - 22/OCT/2015
  ;        o Replaced some string operations with the command StringField for extra speed.
  ;        o Replaced some IFs with SELECT+CASE for extra speed.
  ; V2.5 - 18/JAN/2016
  ;        o Decoding algorithm improved with minor speed gain (it now uses chr(1) INSTEAD of ",," in the ultrafast array
  ; V2.6 - 10/APR/2016
  ;        o Fix: While editing words, more than one prefix would show an incorrect "Code Position"
  ; V2.7 - 13/MAR/2017
  ;        o Added decoding support for (no recursivity yet): 1) FLAG long  2) FLAG num
  ; V2.8 - 28/MAR/2017
  ;        o Added recursivity to dictionary word decoding.
  ; V2.9 - 30/MAR/2017
  ;        o Improved recursivity to dictionary word decoding.  
  ; V3.0 - 31/MAR/2017
  ;        o Improved dictionary word decoding (recursivity)
  ; V3.1 - 1/APR/2017
  ;        o Improved dictionary word decoding (recursivity)  
  ; V3.2 - 4/APR/2017
  ;        o Support via coloured symbol for NOSUGGEST in words editor    
  ;        o Improved: If a code isn't found in the .aff it no longer exits the decoding function
  ; V3.3 - 16/APR/2017
  ;        o Fixed another prefix bug with twofold
  ; V3.4 - 28/APR/2017
  ;        o Support for extracting wordlist compounds for LanguageTool and check for duplicates makes use of morphological information
  ; V3.5 - 21/NOV/2017
  ;        o Added "Missing Codes:" to the Dictionary editor.
  ; V3.6 - 28/NOV/2017
  ;        o Fix: NOSUGGEST flag no longer appears in the not found codes.
  ; V3.7 - 3/DEC/2017
  ;        o Fix: Decoding of rules with a 0 (pt_PT)
  ; V3.8 - 1/APR/2018
  ;        o Hyphenation support
  ; V3.9 - 13/APR/2018
  ;        o Support for compressed AF  
  ; V4.0 - 16/APR/2018
  ;        o NEEDAFFIX support - 16/APR/2018
  ; V4.1 - 27/APR/2018
  ;        o Major Speed up of dictionary processing
  ; V4.2 - 10/MAY/2018
  ;        o Fix: Coded "SFX N" rules, by not applying prefixes to it.
  ; V4.3 - 15/MAY/2018
  ;        o Speed up in dictionary processing by searching first for "SFX Y" instead of "SFX N"
  ; V4.4 - 18/MAY/2018
  ;        o Taboo's warning sign is now an image and is stored in a long instead of string for speed up. 
  ;        o New strategy for extra speed decoding the dictionary, but it takes more RAM.
  ; V4.5 - 23/MAY/2018
  ;        o Fix: Decoding of prefixes wasn't working for some suffixes under certain conditions (due to the new "SFX N" code)
  ; V4.6 - 7/JUN/2018
  ;        o Cleaned the decoding of prefixes and suffixes
  ;
  Procedure process_pfx_sfx(type_of_processing_of_pfx_sfx,word$,csv,compounds,recursivity,languagetool$,morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)
    
    ; NOSUGGEST support - 4/APR/2017
    If recursivity=#False
      nosuggestword_colour=#Green
    EndIf
    
    
    ; NEEDAFFIX support - 16/APR/2018
    If recursivity=#False
      needaffixword_code=#False
    EndIf    
    
    
    ; FORBIDDENWORD support - 14/JUN/2018
    If recursivity=#False
      forbiddenword_code=#False
    EndIf    
        
    
    ; MISSING CODES - 21/NOV/2017
    missing_codes$=""
    
    
    ; COMPRESSED AF - 13/APR/2018
    If AF_compressed_number>0 And AF_compressed_decoding_done=#False
      If Val(StringField(word$,2,"/"))>0 And Val(StringField(word$,2,"/"))<=AF_compressed_number
        word$=StringField(word$,1,"/")+"/"+AF_compressed_array$(Val(StringField(word$,2,"/")))
      EndIf
      AF_compressed_decoding_done=#True
    EndIf
    
    
    ; NEEDAFFIX support - 16/APR/2018
    ; It checks all word codes to look for the NEEDAFIX
    If needaffixword=#True And needaffixword_code=#False
      
      t$=word$
      t$=ReplaceString(t$,"\/",";") ; Support escaped - 15/NOV/2014
      t=FindString(t$,"/")
      t$=ReplaceString(t$,";","/") ; Support escaped - 15/NOV/2014
      tf=t ; 31/JUL/2014 - To be used inside the loops to avoid repeating FINDSTRING(T$,"/")
        
      If Right(t$,1)<>"/" And tf>0
        word_codes$=Mid(t$,t+1) ; Get the codes after each word 11/AUG/2013 - Removed the len for extra speed - 22/OCT/2015

        ; For utf-8/chr,long AND num - 13/MAR/2017
        len_word_codes=Len(word_codes$)  
        If flag_char=2
          len_word_codes=Round(Len(word_codes$)/2.0,#PB_Round_Up)
        EndIf
        If flag_char=3
          len_word_codes=CountString(word_codes$,",")+1
        EndIf         
        
      EndIf
     
      
      For f=1 To len_word_codes ;Len(word_codes$)                        
            
            
        ; 10/DEC/2014 - tried to optimise slightly by only getting the variable value once of Mid(word_codes$,f,1)
        faster_word_codes$=Mid(word_codes$,f,1)
        ; 13/MAR/2017 - support for long and num
        If flag_char=2
          faster_word_codes$=Mid(word_codes$,(f-1)*2+1,2)
        EndIf
        If flag_char=3
          faster_word_codes$=StringField(word_codes$,f,",")
        EndIf
            
            
        ; NEEDAFFIX support - 16/APR/2018
        If faster_word_codes$=needaffixword$ And needaffixword$<>""
          needaffixword_code=#True
          Break
        EndIf   

      Next f
            
    EndIf
    
    
 Debug "word$:"+word$  
    If recursivity=#False
        ; Variables below to handle the combination of prefixes with suffixes+primary
        ; 8/DEC/2014
        Dim primary_suffixes$(0) ; Array that stores the words primary+suffixes
        Dim primary_suffixes_sfx_N(0) ; Array that stores the words primary+suffixes ;If =1 it doesn't apply PREFIXES SFX N
        flag_sfx_no=0 ; To not derivates prefixes for given word - 10/MAY/2018: 0=No,1=Yes  
        total_primary_suffixes=0 ; total of primary+suffixes
        Dim prefixes_codes$(0)   ; Array that stores the CODES of the prefixes
        Dim prefixes_codes_position(0); Array that stores the POSITIONS of the CODES of the prefixes
        Dim prefixes_codes_position_in_array(0) ; to store the position of prefix in the array, not in memory
        total_prefixes_codes=0 ; Total of prefixes CODES
    EndIf  

    
        If recursivity=#False : total_words_found=0 : EndIf ; To use in ProcedureReturn
        t$=word$
        t$=ReplaceString(t$,"\/",";") ; Support escaped - 15/NOV/2014
        t=FindString(t$,"/")
        t$=ReplaceString(t$,";","/") ; Support escaped - 15/NOV/2014
        tf=t ; 31/JUL/2014 - To be used inside the loops to avoid repeating FINDSTRING(T$,"/")
        t2$=t$ : If t>1 : t2$=Left(t$,t-1) : EndIf ; t2$ holds the word before the "/" to simplify the decoding
        If t2$="" : ProcedureReturn 0 : EndIf      ; empty word
        
        
If recursivity=#False        
      
        Select type_of_processing_of_pfx_sfx
         Case 1        
           If hyphenate=#False
             If needaffixword_code=#False
               AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,t2$+#LF$+"Primary")
               SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,0,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_primary),1)
             EndIf
           Else
             If needaffixword_code=#False : AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,t2$+#LF$+"") : EndIf
           EndIf
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no 
;           If needaffixword_code=#False
            total_primary_suffixes+1                            
If needaffixword_code=#False
            total_words_found+1
          EndIf
          
         Case 2        
          If csv=#False And compounds=#False And needaffixword_code=#False: WriteStringN(1,t2$,#PB_UTF8) : EndIf
          If csv=#False And compounds=#True And FindString(t2$,"-") And needaffixword_code=#False: WriteStringN(1,t2$+languagetool$,#PB_UTF8) : total_words_found+1 : EndIf
          If csv=#True And needaffixword_code=#False : WriteStringN(1,Chr(34)+t2$+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+Chr(34)+"Primary"+Chr(34)+";"+Chr(34)+Chr(34)+";"+Chr(34)+Chr(34)+";"+Chr(34)+Chr(34),#PB_UTF8) : EndIf          
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no          
;           If needaffixword_code=#False
            total_primary_suffixes+1                  
If needaffixword_code=#False            
            If compounds=#False : total_words_found+1 : EndIf
          EndIf
          
         Case 3 
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no          
;           If needaffixword_code=#False
            total_primary_suffixes+1                            
If needaffixword_code=#False
            total_words_found+1
          EndIf
          
         Case 4
          ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
          total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=t2$+morphologic_part$+lines_part$
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no           
;           If needaffixword_code=#False
            total_primary_suffixes+1                            
If needaffixword_code=#False
            total_words_found+1
          EndIf
          
         Case 5 
          If csv=#False : process_statistics(t2$): EndIf
If needaffixword_code=#False          
          total_words_found+1
EndIf
          ; 8/DEC/2014
          ReDim primary_suffixes$(total_primary_suffixes)        
          primary_suffixes$(total_primary_suffixes)=t2$
          ; 10/MAY/2018
          ReDim primary_suffixes_sfx_N(total_primary_suffixes)        
          primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no 
;           If needaffixword_code=#False
            total_primary_suffixes+1                  
;           EndIf
          
        EndSelect
        
EndIf        
        
            
        If Right(t$,1)<>"/" And tf>0
          word_codes$=Mid(t$,t+1) ; Get the codes after each word 11/AUG/2013 - Removed the len for extra speed - 22/OCT/2015
          
              
          If type_of_processing_of_pfx_sfx=1 And hyphenate=#False : AddWindowTimer(#WINDOW_ADD_EDIT,800,800) : EndIf
          
          ; For utf-8/chr,long AND num - 13/MAR/2017
          len_word_codes=Len(word_codes$)  
          If flag_char=2
            len_word_codes=Round(Len(word_codes$)/2.0,#PB_Round_Up)
          EndIf
          If flag_char=3
            len_word_codes=CountString(word_codes$,",")+1
          EndIf
          
          
Debug "flag_char:"+Str(flag_char)
Debug "word_codes$:"+word_codes$
Debug "len_word_codes:"+Str(len_word_codes)


          For f=1 To len_word_codes ;Len(word_codes$)                        
            
            flag_sfx_no=0 ; To not derivates prefixes for given word - 10/MAY/2018: 0=No,1=Yes                        
            
            ; 10/DEC/2014 - tried to optimise slightly by only getting the variable value once of Mid(word_codes$,f,1)
            faster_word_codes$=Mid(word_codes$,f,1)
            ; 13/MAR/2017 - support for long and num
            If flag_char=2
              faster_word_codes$=Mid(word_codes$,(f-1)*2+1,2)
            EndIf
            If flag_char=3
              faster_word_codes$=StringField(word_codes$,f,",")
            EndIf
Debug "faster_word_codes$:"+faster_word_codes$          


            ; NOSUGGEST support - 4/APR/2017
            If faster_word_codes$=nosuggestword$ And nosuggestword$<>""
                nosuggestword_colour=#Red
            EndIf
            
            
            ; FORBIDDENWORD support - 4/APR/2017
            If faster_word_codes$=forbiddenword$ And forbiddenword$<>""
              forbiddenword_code=#True              
            EndIf
            
   
            ; 12/JUL/2014 - Code optimised: "N" is first searched and only if NOT found, it searches for "Y"
            ; 17/JAN/2015 - Optimised the code the the ultrafast technique
            pfx=0 ;To have one routine instead of two, because it can't be a PFX and a SFX at the same time - 31/OCT/2013: IT CAN NOW BE A PREFIX AND SUFFIX AT THE SAME TIME
            sfx=0 ;To have one routine instead of two, because it can't be a PFX and a SFX at the same time - 31/OCT/2013: IT CAN NOW BE A PREFIX AND SUFFIX AT THE SAME TIME
            position1=0 ; PFX
            position2=0 ; SFX               
            t1=FindString(ultrafast_aff_search$,"PFX "+faster_word_codes$+" Y ")              
            If t1<>0 : pfx=1 : position1=rules_of_words_total_direct_memory_access(t1) : EndIf  
            t1=FindString(ultrafast_aff_search$,"SFX "+faster_word_codes$+" Y ") ; Now gives priority to Y - 15/MAY/2018
            If t1=0
              t1=FindString(ultrafast_aff_search$,"SFX "+faster_word_codes$+" N ")
              If t1<>0: flag_sfx_no=1 : EndIf              
            EndIf                                  
            If t1<>0 : sfx=1 : position2=rules_of_words_total_direct_memory_access(t1) : EndIf                                              
            ; Line below REMed 4/APR/2017 - Improved: If a code isn't found in the .aff it no longer exits the decoding function
;             If pfx=0 And sfx=0 : Debug "not found" : Break : EndIf ;Nothing found in the .AFF file
            ;ultrafast technique, get the real position from the ultrafast string if position1>0 or position2>0
            If pfx=0 And sfx=0 And faster_word_codes$<>nosuggestword$ And faster_word_codes$<>needaffixword$ And faster_word_codes$<>forbiddenword$ : missing_codes$+faster_word_codes$ : EndIf
            
                
; ADD PREFIXES HERE NEW CODE             
            
            ; Support for LONG and NUM - 13/MAR/2017
            add_flag_length=0
            If flag_char=2 : add_flag_length=1 : EndIf
            If flag_char=3 : add_flag_length=Len(faster_word_codes$)-1 : EndIf            
            
            
            ; 8/DEC/2014
            ; Get the PFX codes and add them + the position to an array,
            ; for at the end combine PFX with PRIMARY+SFXs
            If pfx=1
              
              
              ; 8/DEC/2014
;               Debug "total_prefixes_codes:"+Str(total_prefixes_codes)+":"+Mid(word_codes$,f,1)
              ReDim prefixes_codes$(total_prefixes_codes)              
              ReDim prefixes_codes_position(total_prefixes_codes)       
              ReDim prefixes_codes_position_in_array(total_prefixes_codes)
              prefixes_codes$(total_prefixes_codes)=faster_word_codes$
              prefixes_codes_position(total_prefixes_codes)=rules_of_words_position(position1)
              prefixes_codes_position_in_array(total_prefixes_codes)=position1
              total_prefixes_codes+1
              
            EndIf
    
            
            
            ; Get the number of lines per SFX - 31/OCT/2013
            ; Now ultra-fast - just 2 lines instead of 10 - 12/JUL/2014
            If sfx=1 ; And flag_sfx_no<=1
              
;               If flag_sfx_no=1 : flag_sfx_no=2 : EndIf
                            
              number_of_rules=rules_of_words_total(position2)
              
;  Debug "The SFX has # of meanings:"+a$
              If number_of_rules=0 : Break : EndIf ;To check for a possible zero number of conditions

              For ff=1 To number_of_rules
                  
                ; Fix to window stop responding if we click on it while in the loop
                If type_of_processing_of_pfx_sfx=1 
                  event = WindowEvent()
                  If Event = #PB_Event_Timer And EventTimer() = 800 : EndIf                          
                EndIf
                  
                ; This is each group of text with the conditions, up to five
                ; Moved to this position on 24/AUG/2013 since the guide says that using Dim on an existing array, resets it,
                ; which is faster than reseting it with a FOR loop.
                ; Now around 7x faster with the new code - 13/JUL/2014 
                ; Now ultrafast with arrays - 27/APR/2018
                b$=rules_of_words_each$(position2,ff)
Debug "BBBBBBBBBBBBBB$$$$$$$$$$:"+b$
                ; Removed the two lines below because they aren't used in practice                  
;                 conditions_per_line_sfx$(1)=Left(b$,3)
;                 conditions_per_line_sfx$(2)=Mid(b$,5,1)                                    
                ; Replaced the FOR loop using MID$ and FINDSTRING$ with STRINGFIELD for extra speed - 22/OCT/2015
                conditions_per_line_sfx$(3)=StringField(b$,3," ")
                conditions_per_line_sfx$(4)=StringField(b$,4," ")
                If conditions_per_line_sfx$(4)="0" : conditions_per_line_sfx$(4)="" : EndIf 
                conditions_per_line_sfx$(5)=StringField(b$,5," ")

                
     
;******************************                  
; PLACE HERE THE CODE TO ADD ITEMS TO THE LISTBOX
; EXAMPLE: SFX S y ies [^aeiou]y
;           1  2 3  4      5
; EXAMPLE: SFX p ão ões ão
;           1  2  3  4   5
;                 
;                 Debug "t2$:"+t2$ 
;                 Debug "word$:"+word$
              
                ; Add text after the word with exceptions in ^[] - 14/AUG/2013
                ;Debug t$+"->"+conditions_per_line_sfx$(5)
                If validate_sfx(t2$,conditions_per_line_sfx$(5))=#True   

                  If conditions_per_line_sfx$(3)="0" : conditions_per_line_sfx$(3)="" : EndIf
                    
                  Select type_of_processing_of_pfx_sfx
                     Case 1   
                      If hyphenate=#False
                        colour_line_in_affixes_listicongadget+1
                        AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+#LF$+"Suffix"+#LF$+faster_word_codes$+#LF$+Str(rules_of_words_position(position2))+#LF$+Str(ff)+": "+conditions_per_line_sfx$(3)+" "+conditions_per_line_sfx$(4)+" "+conditions_per_line_sfx$(5))
                        SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_suffix),1)
                      Else  
                        AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+#LF$+faster_word_codes$)
                      EndIf
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                                        
                      total_words_found+1

                      
                     Case 2 
                      If csv=#False And compounds=#False : WriteStringN(1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/"),#PB_UTF8) : EndIf
                      If csv=#False And compounds=#True And FindString(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),"-") : WriteStringN(1,StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+languagetool$,#PB_UTF8) : total_words_found+1 : EndIf
                      If csv=#True : WriteStringN(1,Chr(34)+Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4)+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+Chr(34)+"Suffix"+Chr(34)+";"+Chr(34)+faster_word_codes$+Chr(34)+";"+Chr(34)+Str(rules_of_words_position(position2))+Chr(34)+";"+Chr(34)+Str(ff)+": "+conditions_per_line_sfx$(3)+" "+conditions_per_line_sfx$(4)+" "+conditions_per_line_sfx$(5)+Chr(34),#PB_UTF8) : EndIf                          
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                       
                      If compounds=#False : total_words_found+1 : EndIf
                      
                     Case 3 
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                                             
                      total_words_found+1
                      
                     Case 4
                      ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                      total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")+morphologic_part$+lines_part$
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                          
                      total_words_found+1
                      
                     Case 5 
                      If csv=#False : process_statistics(StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")) : EndIf
                      ; 8/DEC/2014
                      ReDim primary_suffixes$(total_primary_suffixes)        
                      primary_suffixes$(total_primary_suffixes)=StringField(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),1,"/")
                      ; 10/MAY/2018
                      ReDim primary_suffixes_sfx_N(total_primary_suffixes)  
                      primary_suffixes_sfx_N(total_primary_suffixes)=flag_sfx_no
                      total_primary_suffixes+1                       
                      total_words_found+1
                      
                  EndSelect

                  
                  ; recursivity            
                  If FindString(Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),"/") And recursivity=#False
                    process_pfx_sfx(type_of_processing_of_pfx_sfx,Left(t2$,Len(t2$)-Len(conditions_per_line_sfx$(3)))+conditions_per_line_sfx$(4),csv,compounds,#True,"",morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)                    
                  EndIf  
;                   If recursivity=#True And f=len_word_codes : ProcedureReturn total_words_found : EndIf ;It must always return a number, not #TRUE nor #FALSE
                  
                  
                EndIf
                    
                    
;******************************
                                    
                Next ff                
             EndIf                   
                
                
              Next f
              If type_of_processing_of_pfx_sfx=1 : RemoveWindowTimer(#WINDOW_MAIN,800) : EndIf
              
            EndIf

          
          
; ADD PREFIXES HERE NEW CODE             
          
          ; Get the number of lines per PFX - 31/OCT/2013
          ; Now ultra-fast - just 2 lines instead of 10 - 12/JUL/2014
          ; 8/DEC/2014 - New code to combine PFX with PRIMARY+SFXs
          If total_prefixes_codes>0 And total_primary_suffixes>0
            For f=0 To ArraySize(prefixes_codes_position()) ;total_prefixes_codes-1 ; 16/APR/2017 - <- Fix "twofold_Mauro 5.dic" <- Prefixes
              For f_primary_sfx=0 To total_primary_suffixes-1
                
                number_of_rules=rules_of_words_total(prefixes_codes_position_in_array(f))
;  Debug "The PFX has # of meanings:"+a$
                If number_of_rules=0 : Break : EndIf ;To check for a possible zero number of conditions

                For ff=1 To number_of_rules
                  
                  ; Fix to window stop responding if we click on it while in the loop
                  If type_of_processing_of_pfx_sfx=1 
                    event = WindowEvent()
                    If Event = #PB_Event_Timer And EventTimer() = 800 : EndIf                          
                  EndIf
                  
                  ; This is each group of text with the conditions, up to five
                  ; Moved to this position on 24/AUG/2013 since the guide says that using Dim on an existing array, resets it,
                  ; which is faster than reseting it with a FOR loop.
                  ; Now around 7x faster with the new code - 13/JUL/2014
                  ; Now ultrafast with arrays - 27/APR/2018
                  b$=rules_of_words_each$(prefixes_codes_position_in_array(f),ff)

                  ; Removed the two lines below because they aren't used in practice
;                   conditions_per_line_pfx$(1)=Left(b$,3)
;                   conditions_per_line_pfx$(2)=Mid(b$,5,1)                                    
                  ; Replaced the FOR loop using MID$ and FINDSTRING$ with STRINGFIELD for extra speed - 22/OCT/2015
                  conditions_per_line_pfx$(3)=StringField(b$,3," ")
                  conditions_per_line_pfx$(4)=StringField(b$,4," ")
                  If conditions_per_line_sfx$(4)="0" : conditions_per_line_sfx$(4)="" : EndIf 
                  conditions_per_line_pfx$(5)=StringField(b$,5," ")

                  
                 

;******************************                  
; PLACE HERE THE CODE TO ADD ITEMS TO THE LISTBOX
; EXAMPLE: SFX S y ies [^aeiou]y
;           1  2 3  4      5
; EXAMPLE: SFX p ão ões ão
;           1  2  3  4   5
;
                  ; 8/DEC/2014
                  ; Get the primary+suffixes words from the stored array
                  t2$=primary_suffixes$(f_primary_sfx)
                  
                  
                  ; Add text after the word with exceptions in ^[] - 17/AUG/2013
                  ;Debug t$+"->"+conditions_per_line_pfx$(5)
                 If primary_suffixes_sfx_N(f_primary_sfx)=0 ; 10/MAY/2018 - To comply with SFX N
                  If validate_pfx(t2$,conditions_per_line_pfx$(5))=#True                                                                                                
                                        
                    If conditions_per_line_pfx$(3)<>"0"
                                            
                      Select type_of_processing_of_pfx_sfx
                       Case 1                      
                        If hyphenate=#False 
                          colour_line_in_affixes_listicongadget+1
                          AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+#LF$+"Prefix"+#LF$+prefixes_codes$(f)+#LF$+Str(prefixes_codes_position(f))+#LF$+Str(ff)+": "+conditions_per_line_pfx$(3)+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5))
                          SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)
                        Else
                          AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+#LF$+prefixes_codes$(f))
                        EndIf
                        total_words_found+1
                        
                       Case 2 
                        If csv=#False And compounds=#False: WriteStringN(1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/"),#PB_UTF8) : EndIf
                        If csv=#False And compounds=#True And FindString(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),"-") : WriteStringN(1,StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+languagetool$,#PB_UTF8) : total_words_found+1 : EndIf
                        If csv=#True : WriteStringN(1,Chr(34)+conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3)))+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+"Prefix"+Chr(34)+";"+Chr(34)+prefixes_codes$(f)+Chr(34)+";"+Chr(34)+Str(prefixes_codes_position(f))+Chr(34)+";"+Chr(34)+Str(ff)+Chr(34)+";"+Chr(34)+": "+conditions_per_line_pfx$(3)+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5)+Chr(34),#PB_UTF8) : EndIf                          
                        If compounds=#False : total_words_found+1 : EndIf
                        
                       Case 3                       
                        total_words_found+1
                        
                       Case 4 
                        ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                        total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")+morphologic_part$+lines_part$
                        total_words_found+1
                        
                       Case 5
                        If csv=#False : process_statistics(StringField(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),1,"/")) : EndIf
                        total_words_found+1
                        
                      EndSelect
                      
                      
                    Else                        
                      
                      conditions_per_line_pfx$(3)=""
                      
                      Select type_of_processing_of_pfx_sfx
                       Case 1                      
                        If hyphenate=#False 
                          colour_line_in_affixes_listicongadget+1
                          AddGadgetItem(#PANEL_BOX_DERIVED_WORDS,-1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+#LF$+"Prefix"+#LF$+prefixes_codes$(f)+#LF$+Str(prefixes_codes_position(f))+#LF$+Str(ff)+": "+"0"+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5))
                          SetGadgetItemColor(#PANEL_BOX_DERIVED_WORDS,colour_line_in_affixes_listicongadget,#PB_Gadget_FrontColor,affix_colour_line(affix_colour_prefix),1)    
                        Else
                          AddGadgetItem(#LISTICONGADGET_PANEL_HYPHENATOR_DIC,-1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+#LF$+prefixes_codes$(f))
                        EndIf
                        total_words_found+1
                        
                       Case 2
                        If csv=#False And compounds=#False : WriteStringN(1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/"),#PB_UTF8) : EndIf
                        If csv=#False And compounds=#True And FindString(conditions_per_line_pfx$(4)+t2$,"-") : WriteStringN(1,StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+languagetool$,#PB_UTF8) : total_words_found+1 : EndIf
                        If csv=#True : WriteStringN(1,Chr(34)+conditions_per_line_pfx$(4)+t2$+Chr(34)+";"+Chr(34)+t2$+Chr(34)+";"+Chr(34)+"Prefix"+Chr(34)+";"+Chr(34)+prefixes_codes$(f)+Chr(34)+";"+Chr(34)+Str(prefixes_codes_position(f))+Chr(34)+";"+Chr(34)+Str(ff)+": "+"0"+" "+conditions_per_line_pfx$(4)+" "+conditions_per_line_pfx$(5)+Chr(34),#PB_UTF8) : EndIf
                        If compounds=#False : total_words_found+1 : EndIf
                        
                       Case 3 
                        total_words_found+1
                        
                       Case 4                       
                        ReDim total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)
                        total_number_of_processed_words_array$(total_number_of_processed_words+total_words_found)=StringField(conditions_per_line_pfx$(4)+t2$,1,"/")+morphologic_part$+lines_part$
                        total_words_found+1
                        
                       Case 5                      
                        If csv=#False : process_statistics(StringField(conditions_per_line_pfx$(4)+t2$,1,"/")) : EndIf
                        total_words_found+1
                        
                      EndSelect
                      
                      
                    EndIf
                    
                    ; 1/APR/2017 - <- Fix "twofold_Mauro 4.dic" <- Prefixes
                    ; 16/APR/2017 - <- Fix "twofold_Mauro 5.dic" <- Prefixes - (nothing changed here, but above)
                    Debug "recursivity:"+Str(recursivity)+" total_prefixes_codes:"+Str(total_prefixes_codes)
                    If recursivity=#True
                      If total_prefixes_codes>0 : total_prefixes_codes-1 : EndIf                                            
                    EndIf                      
                    
                    ; recursivity            
                    If FindString(conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),"/") And recursivity=#False                      
                      process_pfx_sfx(type_of_processing_of_pfx_sfx,conditions_per_line_pfx$(4)+Right(t2$,Len(t2$)-Len(conditions_per_line_pfx$(3))),csv,compounds,#True,"",morphologic_part$,lines_part$,hyphenate,AF_compressed_decoding_done)                      
                    EndIf                      
;                     If recursivity=#True And f=len_word_codes : ProcedureReturn total_words_found : EndIf ;It must always return a number, not #TRUE nor #FALSE
                    
                    
                  EndIf                     
                  
                EndIf
;******************************
                  
                Next ff
              Next f_primary_sfx
             Next f
          EndIf                   
          
;  Debug "Refresh derived words"

          
        ; Missing Codes - 21/NOV/2017
        If IsGadget(#STRING_MISSING_CODES)
          SetGadgetText(#STRING_MISSING_CODES,missing_codes$)
          If missing_codes$=""
            SetGadgetColor(#STRING_MISSING_CODES,#PB_Gadget_BackColor,-1)
          Else
            SetGadgetColor(#STRING_MISSING_CODES,#PB_Gadget_BackColor,#Red)
          EndIf
        EndIf
        
          
        ProcedureReturn total_words_found ;It must always return a number, not #TRUE nor #FALSE
        
  EndProcedure
  
  
  
  
  ; Validate Suffix to check if letters match/unmatch 
  ; validate_sfx(t$,conditions_per_line_sfx$(5))
  ; V1.0 - 14/AUG/2013
  ; V1.1 - 18/AUG/2013
  ;        o Fixed the procedure
  ; V1.2 - 26/AUG/2013
  ;        o Major speed up decoding suffixes
  ; V1.3 - 29/SEP/2013
  ;        o Improved the code to detect a "." in the rule
  ; V1.4 - 30/SEP/2013
  ;        o Improved the decoding of codes for multiple conditions
  ; V1.5 - 13/JUL/2014
  ;        o Optimised the code
  ; V1.6 - 15/JUL/2014  
  ;        o Rewrote most of the code to fix more complex rules: beast/J -> rule #22 -> [^aeio]e[nrt]
  ; V1.7 - 16/JAN/2016
  ;        o Decoding algorithm improved with minor speed gain
  ;
  Procedure validate_sfx(a$,b$)  
Debug "SFX a$:"+a$+ " b$:"+b$+" conditions_per_line_sfx$(3):"+conditions_per_line_sfx$(3)
    ; Check if the rule ends with "."
    If b$="." : ProcedureReturn #True : EndIf
   
    ; Check if characters to replace don't match the original word - faster technique - 16/01/2016
    If conditions_per_line_sfx$(3)<>"0" And Right(a$,Len(conditions_per_line_sfx$(3)))<>conditions_per_line_sfx$(3) : ProcedureReturn #False : EndIf
   
    ;First find if the end of condition matches word, after the ] or no ]
    c$=""
    t1=FindString(b$,"[")
    If t1=0
      c$=b$
    Else
      c$=Right(b$,FindString(ReverseString(b$),"]")-1)
    EndIf
    If c$="." And Not Len(a$)>=2 : ProcedureReturn #False : EndIf
    If c$="." And Len(a$)>=2 : c$="" : EndIf    
    If c$<>"" And Right(a$,Len(c$))<>c$ : ProcedureReturn #False : EndIf        
;     Debug "a$:"+a$+" b$:"+b$+" c$:"+c$   
 
    ; No more conditions
    If t1=0 : ProcedureReturn #True : EndIf
    
    a$=Left(a$,Len(a$)-Len(c$))
    t=Len(a$)
    cond$=Left(b$,Len(b$)-Len(c$))
    For f=Len(cond$) To 1 Step -1              
      
      d$=Mid(cond$,f,1)
      
      ; Check if condition outside []
      If d$<>"]"
        For ff=f To 1 Step -1
          If Mid(cond$,ff,1)="]" : ff+1 : Break : EndIf          
          If Mid(cond$,ff,1)<>Mid(a$,t,1) : ProcedureReturn #False : EndIf
          t-1
        Next ff
        f=ff
        
        
      Else
        
        ; Found a [] condition
        ttt$=""
        For ff=f To 1 Step -1
          ttt$=Mid(cond$,ff,1)+ttt$
          If Mid(cond$,ff,1)="[" : Break : EndIf
        Next ff
        f=ff

        ; Get the type of condition: ^=different (flag FALSE)
        flag=#False ;Means a "^" (different)
        If Mid(ttt$,2,1)<>"^" : flag=#True : EndIf
        
        ; Get the content from inside the []
        If flag=#False : ttt$=Mid(ttt$,3,Len(ttt$)-3) : EndIf
        If flag=#True : ttt$=Mid(ttt$,2,Len(ttt$)-2) : EndIf

        ; Compare contents
        match=#False ; To check if the conditions match. #FALSE=FAILED => PROC RETURN #FALSE
        For ff=1 To Len(ttt$)
          If flag=#False And Mid(ttt$,ff,1)=Mid(a$,t,1)
;             Debug "match=#False (PROC RET 0)"
            ProcedureReturn #False
          EndIf
          If flag=#True And Mid(ttt$,ff,1)=Mid(a$,t,1)
            match=#True
            Break
          EndIf
        Next ff        
        If flag=#False : match=#True : EndIf
;         Debug "MATCH:"+match        
        If match=#False : ProcedureReturn #False : EndIf

      EndIf
      
      t-1
            
    Next f
                
    
    ; SFX validated successfully
    ProcedureReturn #True
    
  EndProcedure    
 
  
  
  
  ; Validate Prefix to check if letters match/unmatch 
  ; validate_pfx(t$,conditions_per_line_pfx$(5))
  ; V1.0 - 17/AUG/2013
  ; V1.1 - 18/AUG/2013
  ;        o Fixed the procedure  
  ; V1.2 - 29/SEP/2013
  ;        o Improved the code to detect a "." in the rule
  ; V1.3 - 30/SEP/2013
  ;        o Improved the decoding of codes for multiple conditions
  ; V1.4 - 13/JUL/2014
  ;        o Optimised the code
  ; V1.5 - 16/JAN/2016
  ;        o Decoding algorithm improved with minor speed gain
  ;
  Procedure validate_pfx(a$,b$)
 Debug "PFX a$:"+a$+ " b$:"+b$+" conditions_per_line_pfx$(3):"+conditions_per_line_pfx$(3)   
    ;Check if the rule ends with "."
    If b$="." : ProcedureReturn #True : EndIf
    
    ; Check if characters to replace don't match the original word - faster technique - 16/01/2016
    If conditions_per_line_pfx$(3)<>"0" And Left(a$,Len(conditions_per_line_pfx$(3)))<>conditions_per_line_pfx$(3) : ProcedureReturn #False : EndIf
    
    
    ;First find if the end of condition matches word, after the ] or no ]
    c$=""
    t1=FindString(b$,"[")
    If t1=0
      c$=b$
    Else
      c$=Right(b$,FindString(ReverseString(b$),"]")-1)
    EndIf
    If c$="." And Not Len(a$)>=2 : ProcedureReturn #False : EndIf
    If c$="." And Len(a$)>=2 : c$="" : EndIf
    If c$<>"" And Left(a$,Len(c$))<>c$ : ProcedureReturn #False : EndIf
    ccc$=c$
;Debug "a$:"+a$+" b$:"+b$+" c$:"+c$
    
    ; Find the letters inside a [^] and [] and validate them
    t=1
    t1=CountString(b$,"]")
    If t1<>0
      For f=t1 To 1 Step -1
        c$=""
        t=FindString(b$,"[",t)
        t+1
        
        If Mid(b$,t,1)="^"
          c$=Mid(b$,t+1,FindString(b$,"]",t)-t-1)
          If c$<>""
            For ff=1 To Len(c$)
;              Debug "mid:"+Mid(a$,Len(ccc$)+f-1+1,1)+" f:"+Str(f)
              If Mid(a$,Len(ccc$)+f-1+1,1)=Mid(c$,ff,1) : ProcedureReturn #False : EndIf
            Next ff
          EndIf          
        EndIf
        
        If Mid(b$,t,1)<>"^"
          c$=Mid(b$,t,FindString(b$,"]",t)-t)
          If c$<>""
            flag=#False
            For ff=1 To Len(c$)
;              Debug "mid:"+Mid(a$,Len(ccc$)+f-1+1,1)+" f:"+Str(f)              
              If Mid(a$,Len(ccc$)+f-1+1,1)=Mid(c$,ff,1)
                flag=#True
                Break
              EndIf
            Next ff
            If flag=#False : ProcedureReturn #False : EndIf
          EndIf                   
        EndIf
        
      Next f
    EndIf
    
    ; PFX validated successfully
    ProcedureReturn #True
    
  EndProcedure  
  
  
  
  
  ; To update the vertical scroll bar of the dictionary and refresh the items in the listicongadget
  ; V1.0 - 27/FEB/2014
  ; V1.1 - 28/FEB/2014
  ;        o Now the procedure returns if the number of words is zero, just after clearing the items in the window
  ; V1.2 - 5/MAR/2014
  ;        o It now scrolls okay with selected words
  ; V1.3 - 23/MAR/2014
  ;        o It now resets the vertical scroll bar if number of words=0
  ; V1.4 - 10/FEB/2016
  ;        o The dictionary now shows line numbers.
  ; V1.5 - 12/FEB/2016
  ;        o Improved the line numbers grey colour in Linux.
  ; V1.6 - 29/SEP/2016
  ;        o Dynamic number of lines  
  ; V1.7 - 22/DEC/2016
  ;        o Started implementing the multiselection.  
  ;
  Procedure update_vertical_scroll_bar_dictionary()  
    ClearGadgetItems(#PANEL_BOX_DICTIONARY)
    If number_of_words=0
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Minimum,0)
      SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,0)
      ProcedureReturn
    EndIf
    beginloop = GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
;     endloop = beginloop + 10
    endloop=beginloop+number_of_lines-1
    If endloop>ArraySize(words$())
      endloop=ArraySize(words$())
    EndIf
    For f=beginloop To endloop      
      AddGadgetItem(#PANEL_BOX_DICTIONARY,-1,Str(f+1)+#LF$+words$(f))      
      SetGadgetItemColor(#PANEL_BOX_DICTIONARY,#PB_All,#PB_Gadget_FrontColor,$999999,0)
      If words_selected(f)=#True : SetGadgetItemState(#PANEL_BOX_DICTIONARY,f-beginloop,#PB_ListIcon_Checked) : EndIf
    Next f
    
    t=beginloop
    t1=selected_dictionary_word-t
;     If t1>=0 And t1<=10 : SetGadgetState(#PANEL_BOX_DICTIONARY,t1) : EndIf           
    If t1>=0 And t1<=number_of_lines-1 : SetGadgetState(#PANEL_BOX_DICTIONARY,t1) : EndIf           
    
  EndProcedure
  
  
  
  
  ; To always have the current word highlighted in the listicongadget
  ; V1.0 - 5/MAR/2014
  ;
  Procedure update_vertical_scroll_bar_dictionary_current_item()  
    If selected_dictionary_word=-1 And GetGadgetState(#PANEL_BOX_DICTIONARY)=-1 : ProcedureReturn #False : EndIf
  
    selected_dictionary_word=GetGadgetState(#PANEL_BOX_DICTIONARY)+GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)   
    t=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
    t1=selected_dictionary_word-t
    
  EndProcedure
  
  
  

; To extract the wordlist to HDD
; Please notice that the process_pfx_sfx procedure will save directly to HDD
; Variable EXTENSION:
;                    0=.TXT
;                    1=.CSV
;                    2=COMPOUNDS .TXT
;
; V1.0 - 11/JUL/2014
; V1.1 - 12/JUL/2014
;        o The tags part of the word wasn't removed
; V1.2 - 16/JUL/2014
;        o Added CSV
; V1.3 - 17/JUL/2014
;        o In CSV it now has an extra field with the primary word and the separator is now ";" instead of ",".
; V1.4 - 31/JUL/2014
;        o The option to extract the wordlist now has TXT and CSV separately to be compatible with MacOS X.
; V1.5 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.6 - 5/MAR/2015
;        o Coded "Extract Compounds .txt".
; V1.7 - 28/APR/2016
;        o Support for LanguageTool
; V1.8 - 25/OCT/2017
;        o Formats the total number of items to be better seen.  
; V1.9 - 1/DEC/2017
;        o Message Requesters now have a sign. 
;
Procedure extract_wordlist(extension)
    Debug "Pressed EXTRACT WORDLIST"
    
    languagetool$=""
    If extension=6
      languagetool$="*"  ; Extract compounds in LanguageTool format
      extension=2
    EndIf
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting wordlist... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()

       
    t$="Text Documents (*.txt)|*.txt"
    If extension=1 : t$="Text CSV (*.csv)|*.csv" : EndIf
    tt$="Extract wordlist to file"
    If extension=2 : tt$="Extract Compounds to file" : EndIf
    file$=SaveFileRequester(tt$,"",t$,0)
    If file$=""
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    If (extension=0 Or extension=2) And LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
    If extension=1 And LCase(Right(file$,4))<>".csv" : file$+".csv": EndIf
    If extension=1 : csv=#True : EndIf
    If extension=2 : compounds=#True : EndIf
    
    
    t=ReadFile(1,file$)
    If t<>0
      CloseFile(1)
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    DeleteFile(file$) ; Delete .TXT file
    
    
    ; Saving the .TXT file
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,file$)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
      time=ElapsedMilliseconds()
      FUCKTIME=ElapsedMilliseconds()
      total_number_of_processed_words=0
      
      ; Write CSV header
      If csv=#True
        WriteStringN(1,Chr(34)+"Word"+Chr(34)+";"+Chr(34)+"Primary"+Chr(34)+";"+Chr(34)+"Type"+Chr(34)+";"+Chr(34)+"Code"+Chr(34)+";"+Chr(34)+"Position"+Chr(34)+";"+Chr(34)+"Rule"+Chr(34),#PB_UTF8)
      EndIf
      
      For f=0 To number_of_words-1
        text$=StringField(words$(f),1,#LF$)
        
        total_number_of_processed_words+process_pfx_sfx(2,text$,csv,compounds,#False,languagetool$,"","",#False,#False)
;         WriteStringN(1,text$,#PB_UTF8)
                      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Wordlist... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        

      Next f
      Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      CloseFile(1)
      
      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay. Words Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
     
    
EndProcedure




; To count the wordlist
; Please notice that the process_pfx_sfx procedure will return the number of words decoded (total)
; V1.0 - 11/JUL/2014
; V1.1 - 12/JUL/2014
;        o The tags part of the word wasn't removed
; V1.2 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.3 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.4 - 1/DEC/2017
;        o Message Requesters now have a sign.  
; V1.5 - 15/MAY/2018
;        o Fix: Count words now separates thousands with the preferences character.
;
Procedure count_wordlist()
    Debug "Pressed COUNT WORDLIST"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Counting wordlist... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
        
            
    ; Counting wordlist
    AddWindowTimer(#WINDOW_MAIN,800,800)    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    total_number_of_processed_words=0
      
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
        
      total_number_of_processed_words+process_pfx_sfx(3,text$,#False,#False,#False,"","","",#False,#False)
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Counting Wordlist... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
        EndIf
      EndIf
        

      Next f
      Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      
      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay. Words Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
     
    
EndProcedure




; To find duplicates in the wordlist
; V1.0 -  6/AUG/2014
; V1.1 - 29/AUG/2014
;        o Now it gives an error if one attempts to export a blank list
;        o Fix: The number of words text no longer cuts the "Export" button while hidding and then showing PTG.
; V1.2 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.3 -  7/AUG/2015
;        o If "Show Duplicates" finds nothing, it opens a requester saying so.
; V1.4 - 11/MAR/2017
;        o Changed the buttons height to 25. 
; V1.5 - 28/APR/2017
;        o Support for morphological data
; V1.6 - 29/APR/2017
;        o Fix: Check for duplicates improved (morphologic data)
; V1.7 - 20/MAY/2017
;        o Now the check for duplicates in dictionary, also shows the lines numbers.
; V1.8 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.9 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure show_duplicates_wordlist()
    Debug "Pressed SHOW DUPLICATES"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
    
    
            
    ; Step 1 of 2 (extracting wordlist)
    AddWindowTimer(#WINDOW_MAIN,800,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    total_number_of_processed_words=0
    Dim total_number_of_processed_words_array$(0)
    
    
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
      morphologic_part$=#LF$+StringField(words$(f),2,#LF$)
      lines_part$=#LF$+Str(f+1)
        
      total_number_of_processed_words+process_pfx_sfx(4,text$,#False,#False,#False,"",morphologic_part$,lines_part$,#False,#False)
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting..."+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" <ESC> to abort.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; Sort the array    
    SortArray(total_number_of_processed_words_array$(),#PB_Sort_Ascending)    
    
    
    ; Step 2 of 2 (comparing wordlist)    
    
    ; First open a hidden window
    If OpenWindow(#WINDOW_SHOW_DUPLICATES,0,0,400+20-5+250+10,-100+200-50-20-20+100+100+200-30-30-30+10-20-10,"Show Duplicates",#PB_Window_Invisible|#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    ListIconGadget(#LISTICON_WINDOW_SHOW_DUPLICATES,5,5,250+400,200+100-50+10,"Word",100+100,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_DUPLICATES,1,"Occurrences",100)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_DUPLICATES,2,"Lines",250)
    ButtonGadget(#BUTTON_WINDOW_SHOW_EXPORT,5+50+50+50+0+50,200+100-50+10+10,100,25,"Export")
    ButtonGadget(#BUTTON_WINDOW_SHOW_OKAY,5+50+50+50+50+0+100+10,200+100-50+10+10,100-10-10,25,"Close")
    
    ; Progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing... <ESC> to abort.")    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_processed_words)     
        
        
    For f=0 To total_number_of_processed_words-1
;       t$=total_number_of_processed_words_array$(f)
      t$=StringField(total_number_of_processed_words_array$(f),1,#LF$)+#LF$+StringField(total_number_of_processed_words_array$(f),2,#LF$)
      occurrences=0
      lines$=""
      
      For ff=f To total_number_of_processed_words-1            
                
        
        If t$=StringField(total_number_of_processed_words_array$(ff),1,#LF$)+#LF$+StringField(total_number_of_processed_words_array$(ff),2,#LF$) ;=total_number_of_processed_words_array$(ff)          
          occurrences+1          
          If occurrences>1 : lines$+"+" : EndIf
          lines$+FormatNumber(Val(StringField(total_number_of_processed_words_array$(ff),3,#LF$)),0,".",number_separator_chr$)
        Else
          Break
        EndIf
                             
      Next ff
      
      f=ff-1
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
         StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+" <ESC> to abort.")    
         StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          CloseWindow(#WINDOW_SHOW_DUPLICATES)
          ProcedureReturn #False
        EndIf
      EndIf      
      
      
      ; Add duplicate word to listicongadget
      If occurrences>1
          AddGadgetItem(#LISTICON_WINDOW_SHOW_DUPLICATES,-1,StringField(t$,1,#LF$)+#LF$+FormatNumber(occurrences,0,".",number_separator_chr$)+#LF$+lines$)
      EndIf
      
    Next f    
    
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
   
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
    
    
    ; No duplicates found, exit the Procedure - 7/AUG/2015
    If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)=0
      MessageRequester("Warning!","No duplicates found.",#PB_MessageRequester_Warning)
      DisableWindow(#WINDOW_MAIN,#False)
      CloseWindow(#WINDOW_SHOW_DUPLICATES)                
      ProcedureReturn
    EndIf    
    
    
    HideWindow(#WINDOW_SHOW_DUPLICATES,#False)
    DisableWindow(#WINDOW_MAIN,#True)
    SetActiveWindow(#WINDOW_SHOW_DUPLICATES)
    
    ; Add the number of words found
    TextGadget(#TEXT_WINDOW_SHOW_DUPLICATES_FOUND,5+5,200+100-50+10+10,200-20,20+2+1,"Found:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$))
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_DUPLICATES,#PB_Shortcut_Escape,1000)
    
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      
      Select event 
        Case #PB_Event_Gadget
         Select EventGadget()
           Case #BUTTON_WINDOW_SHOW_OKAY
             cancel=#True
             
           Case #BUTTON_WINDOW_SHOW_EXPORT
             If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)>0
               flag=#False
               file$=SaveFileRequester("Export Duplicates to file","","Text Documents (*.txt)|*.txt",0)
               If file$<>""
                 If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
                 t=ReadFile(1,file$)
                 If t<>0
                   CloseFile(1)
                   If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                       DeleteFile(file$) ; Delete .TXT file                 
                   Else
                       flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                   EndIf  
                 EndIf    
                 
                 If flag=#False
                    ; Saving the .TXT file
                    AddWindowTimer(#WINDOW_MAIN,800,800)
                    OpenFile(1,file$)      
                    
                    WriteStringN(1,"DUPLICATES FOUND:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$))
                    WriteStringN(1,"")
                    
                    For f=1 To CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
                      WriteStringN(1,GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)+":"+GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1)+":"+GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,2))
                      
                      ; Fix to window stop responding if we click on it while exporting
                      event = WindowEvent()
                      If event=#PB_Event_Timer And EventTimer()=800 : EndIf 
                
                    Next f
                    CloseFile(1)
                    RemoveWindowTimer(#WINDOW_MAIN,800)          
                 EndIf
                 
               EndIf
               
             EndIf
         EndSelect
         
      EndSelect   
      
    Until cancel=#True
    
    ; Close the Duplicates window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_DUPLICATES)    
    
EndProcedure




; Process the Statistics
;
; V1.0 - 15/DEC/2014
; V1.1 - 17/JAN/2015
;        o Ultrafast searching for rules positions
; V1.2 - 10/APR/2015
;        o Pie chart support
; V1.3 - 13/APR/2015
;        o Improved the pie chart with a message in case it gets above 100% + small fix
; V1.4 - 11/MAR/2017
;        o Changed the buttons height to 25. 
; V1.5 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.6 - 1/DEC/2017
;        o Message Requesters now have a sign.
; V1.7 - 12/JUL/2018
;        o Statistics now allow to calculate the number of new words since the last mentioned number.
; V1.8 - 15/JUL/2018
;        o Statistics now export in Linux format and also the extra information.
; V1.9 - 16/JUL/2018
;        20/JUL/2018
;        o Statistics now autoupdate the number of new words when changing the reference value.
;
Procedure show_statistics()
    Debug "Pressed STATISTICS"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Statistics... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; To create a faster searchable version of the AFF gadgettext with just single spaces
    faster_searchable_version_of_aff()
    
    
    ; Reset the statistics variables
    Global statistics_words_with_apos=0
    Global statistics_words_with_hyphen=0
    Global statistics_words_with_first_letter_uppercase=0
    Global statistics_words_chr1=0
    Global statistics_words_chr2=0
    Global statistics_words_chr3=0
    Global statistics_words_chr4=0
    Global statistics_words_chr5=0
    
    
    ; Processing wordlist
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    total_number_of_processed_words=0
      
      
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
        
      total_number_of_processed_words+process_pfx_sfx(5,text$,#False,#False,#False,"","","",#False,#False)
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Statistics... <ESC> to abort. Found:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
               
      
    ; Open window
    If OpenWindow(#WINDOW_SHOW_STATISTICS,0,0,640-10-10-20-10+3,-30-8+70+3+70-100+200-50-20-20+100+100+200-30-30-30+10-20-10,"Show Statistics",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    ListIconGadget(#LISTICON_WINDOW_SHOW_STATISTICS,5,5,400-50-50+3+5,200+100-50+10,"Statistics (Words)",100+100-50,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS,1,"Occurrences",100)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_STATISTICS,2,"%",40+3+5)    
    CanvasGadget(#CANVAS_WINDOW_SHOW_STATISTICS,4+5+400-50-50+10,5,250+5+5,250+5+5,#PB_Canvas_Border)
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_EXPORT,5+50+50+50+0+50,65+5+200+100-50+10+10+30,100,25,"Export")
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_OKAY,5+50+50+50+50+0+100+10,65+5+200+100-50+10+10+30,100-10-10,25,"Close")
    
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"With ':"+#LF$+FormatNumber(statistics_words_with_apos,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_with_apos*100)/total_number_of_processed_words,1),ImageID(14))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"With Hyphen:"+#LF$+FormatNumber(statistics_words_with_hyphen,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_with_hyphen*100)/total_number_of_processed_words,1),ImageID(15))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"Starting uppercase:"+#LF$+FormatNumber(statistics_words_with_first_letter_uppercase,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_with_first_letter_uppercase*100)/total_number_of_processed_words,1),ImageID(16))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"1 characters long:"+#LF$+FormatNumber(statistics_words_chr1,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_chr1*100)/total_number_of_processed_words,1),ImageID(17))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"2 characters long:"+#LF$+FormatNumber(statistics_words_chr2,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_chr2*100)/total_number_of_processed_words,1),ImageID(18))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"3 characters long:"+#LF$+FormatNumber(statistics_words_chr3,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_chr3*100)/total_number_of_processed_words,1),ImageID(19))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"4 characters long:"+#LF$+FormatNumber(statistics_words_chr4,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_chr4*100)/total_number_of_processed_words,1),ImageID(20))
    AddGadgetItem(#LISTICON_WINDOW_SHOW_STATISTICS,-1,"5 characters long:"+#LF$+FormatNumber(statistics_words_chr5,0,".",number_separator_chr$)+#LF$+StrF((statistics_words_chr5*100)/total_number_of_processed_words,1),ImageID(21))
    
    ; Draw a black circle + %s in the canvas
    StartDrawing(CanvasOutput(#CANVAS_WINDOW_SHOW_STATISTICS))
    Circle(125,125,110,#Black)
    current_degree=0
    For f=0 To 7
      amount_degrees=Int((Val(ReplaceString(GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,f,1),number_separator_chr$,""))*360)/total_number_of_processed_words)
      If amount_degrees>0
        If f=0 : colour_degree=RGB(163,73,164) : EndIf
        If f=1 : colour_degree=RGB(63,72,204) : EndIf
        If f=2 : colour_degree=RGB(0,162,232) : EndIf
        If f=3 : colour_degree=RGB(34,177,76) : EndIf
        If f=4 : colour_degree=RGB(255,242,0) : EndIf
        If f=5 : colour_degree=RGB(255,127,39) : EndIf
        If f=6 : colour_degree=RGB(237,28,36) : EndIf
        If f=7 : colour_degree=RGB(136,0,21) : EndIf
        For ff=current_degree To current_degree+amount_degrees-1
          stepping_degres.f=0
          While stepping_degres<1
            LineXY(125,125,125+110*Cos(Radian(ff+stepping_degres)),125+110*Sin(Radian(ff+stepping_degres)),colour_degree)
            stepping_degres+0.01
          Wend  
        Next ff
        current_degree+amount_degrees
      EndIf
    Next f
    
    ; Check if sum>100.0%
    sum.f=0.0
    For f=0 To 7
      sum+ValF(GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,f,2))
    Next f
    If sum>100.0
      DrawText(5,250+5+5-10-10-2,"NOTE: >100%",#Black,#White)
    EndIf
      
    StopDrawing()
      
    DisableWindow(#WINDOW_MAIN,#True)
    SetActiveWindow(#WINDOW_SHOW_STATISTICS)
    
    ; Add the number of words found
    TextGadget(#TEXT_WINDOW_SHOW_STATISTICS_WORDLIST,5+5,200+100-50+10+10,200-20,20+2+1,"Total wordlist:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$))
    
    x=5+5
    y=GadgetY(#TEXT_WINDOW_SHOW_STATISTICS_WORDLIST)+GadgetHeight(#TEXT_WINDOW_SHOW_STATISTICS_WORDLIST)+5
    
    ; COPY button
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_COPY,x,y,80,25,"∑ Copy →")
    
    ; Remember string
    x+80+10
    StringGadget(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,x,y,100,25,"",#PB_String_Numeric)
    ; Check if remember.PREFS file exists
    If FileSize(main_path$+"remember.prefs")>0
      OpenFile(1,main_path$+"remember.prefs")
        t$=ReadString(1,#PB_UTF8)
      CloseFile(1)
      SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,t$)
    EndIf  
    
    
    ; REMEMBER
    x+100+10
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_REMEMBER,x,y,80,25,"Remember")
    
    ; FORGET
    x+80+10
    ButtonGadget(#BUTTON_WINDOW_SHOW_STATISTICS_FORGET,x,y,80,25,"Forget")    
    
    ; TOTAL NEW
    x=GadgetX(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)
    y=GadgetY(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)+25+5+3
    StringGadget(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,x,y,100+50+10+10+100+5+3+1,25,"",#PB_String_ReadOnly)
    words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
    If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
      SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+"  Date:"+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
    EndIf
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_STATISTICS,#PB_Shortcut_Escape,1000)
 
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      
      Select event 
        Case #PB_Event_Gadget
         Select EventGadget()
           Case #BUTTON_WINDOW_SHOW_STATISTICS_OKAY
             cancel=#True
             
           Case #BUTTON_WINDOW_SHOW_STATISTICS_EXPORT
             flag=#False
             file$=SaveFileRequester("Export Statistics to file","","Text Documents (*.txt)|*.txt",0)
             If file$<>""
               If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
               t=ReadFile(1,file$)
               If t<>0
                 CloseFile(1)
                 If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                     DeleteFile(file$) ; Delete .TXT file                 
                 Else
                     flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                 EndIf  
               EndIf    
                 
               If flag=#False
                  ; Saving the .TXT file
                  OpenFile(1,file$)      
                    
                  WriteString(1,"Total wordlist:"+FormatNumber(total_number_of_processed_words,0,".",number_separator_chr$)+#LF$,#PB_UTF8)
                  WriteString(1,""+#LF$)
                  WriteString(1,"WORDS:"+#LF$)
                  WriteString(1,"With ':"+#LF$+FormatNumber(statistics_words_with_apos,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,0,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"With Hyphen:"+#LF$+FormatNumber(statistics_words_with_hyphen,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,1,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"Starting uppercase:"+#LF$+FormatNumber(statistics_words_with_first_letter_uppercase,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,2,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"1 characters long:"+#LF$+FormatNumber(statistics_words_chr1,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,3,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"2 characters long:"+#LF$+FormatNumber(statistics_words_chr2,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,4,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"3 characters long:"+#LF$+FormatNumber(statistics_words_chr3,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,5,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"4 characters long:"+#LF$+FormatNumber(statistics_words_chr4,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,6,2)+"%)"+#LF$,#PB_UTF8)
                  WriteString(1,"5 characters long:"+#LF$+FormatNumber(statistics_words_chr5,0,".",number_separator_chr$)+" ("+GetGadgetItemText(#LISTICON_WINDOW_SHOW_STATISTICS,7,2)+"%)"+#LF$,#PB_UTF8)    
                  ref_value$="N/A"
                  If GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>"" : ref_value$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER) : EndIf
                  WriteString(1,"Reference Value:"+#LF$+ref_value$+#LF$,#PB_UTF8)
                  new_words$="N/A"
                  If GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW)<>"" : new_words$=Mid(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW),5) : EndIf
                  WriteString(1,"New Words:"+#LF$+new_words$+#LF$,#PB_UTF8)
                  
                  CloseFile(1)
                  
               EndIf
                 
             EndIf
             
             
           ; Manually change number of reference value
           Case #STRING_WINDOW_SHOW_STATISTICS_REMEMBER
             If EventType()=#PB_EventType_Change
               
                ; Update new words
                words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
                If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
                  SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" "+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
                EndIf         
                If GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)="" : SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"") : EndIf
                
             EndIf
             
           ; Copy 
           Case #BUTTON_WINDOW_SHOW_STATISTICS_COPY
             SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,Str(total_number_of_processed_words))
             
             ; Update new words
             words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
             If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
               SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" "+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
             EndIf                  
             
             
           ; Delete old remember.prefs file     
           Case #BUTTON_WINDOW_SHOW_STATISTICS_FORGET
             
              If FileSize(main_path$+"remember.prefs")>=0
                DeleteFile(main_path$+"remember.prefs")              
              EndIf  
            
              SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER,"")
              SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"")
              
              
           ; Remember the previous number of words
           Case #BUTTON_WINDOW_SHOW_STATISTICS_REMEMBER 
             t$=GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)
             flag=#False
             If t$=""
               MessageRequester("Warning!","Remember gadget is blank.",#PB_MessageRequester_Warning)
               flag=#True
             EndIf
             If flag=#False
               
                ; Delete old remember.prefs file    
                If FileSize(main_path$+"remember.prefs")>=0
                  DeleteFile(main_path$+"remember.prefs")
                EndIf                
                
                ; Attempt to save remember.PREFS file
                OpenFile(1,main_path$+"remember.prefs")                
                  WriteString(1,t$+#LF$,#PB_UTF8)
                CloseFile(1)
                
                ; Update new words
                words_remembered=Val(GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER))
                If words_remembered>=0 And GetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_REMEMBER)<>""
                  SetGadgetText(#STRING_WINDOW_SHOW_STATISTICS_TOTAL_NEW,"New:"+FormatNumber(total_number_of_processed_words-words_remembered,0,".",number_separator_chr$)+" "+FormatDate("%yyyy-%mm-%dd %hh:%ii:%ss",Date()))
                EndIf                
                
             EndIf
             
         EndSelect
         
      EndSelect   
      
    Until cancel=#True
    
    ; Close the Duplicates window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_STATISTICS)    
           
EndProcedure




; This procedure is called from inside the process_pfx_sfx code, in order to increase the different type of statistics found
;
; V1.0 - 15/DEC/2014
; V1.1 - 18/JUL/2018
;        o Optimised slightly the statistics processing regarding uppercase words.
;
Procedure process_statistics(t$)
  If FindString(t$,"'")>0 : statistics_words_with_apos+1 : EndIf
  If FindString(t$,"-")>0 : statistics_words_with_hyphen+1 : EndIf
  t1$=Left(t$,1)
  If UCase(t1$)=t1$ : statistics_words_with_first_letter_uppercase+1 : EndIf
  If Len(t$)=1 : statistics_words_chr1+1 : EndIf
  If Len(t$)=2 : statistics_words_chr2+1 : EndIf
  If Len(t$)=3 : statistics_words_chr3+1 : EndIf
  If Len(t$)=4 : statistics_words_chr4+1 : EndIf
  If Len(t$)=5 : statistics_words_chr5+1 : EndIf
EndProcedure




; Fix spaces in dictionary
;
; V1.0 - 16/FEB/2016
; V1.1 - 7/JUN/2017
;        o undo/redo support
; V1.2 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure fix_spaces_wordlist()
    Debug "Pressed FIX SPACES DICTIONARY"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fix Invalid Spaces... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; Processing wordlist - STEP 1 of 2
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
      
    
    word_fixes=0
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        If Left(text$,1)=" " : text$=Mid(text$,2) : EndIf
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And Left(text$,1)<>" " And Right(text$,1)<>" "
      
      ; Abort if an empty word is found
      If text$=""
        Debug "NULL WORD FOUND!"
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Null word at Line:"+Str(f+1)+"! Aborted.")
        RemoveWindowTimer(#WINDOW_MAIN,800)
        ProcedureReturn #False        
      EndIf
      
      
      ; Increment fixes
      If text$<>StringField(words$(f),1,#LF$)
        word_fixes+1        
      EndIf      
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Wordlist 1/2..."+Str(f)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; No fixes found
    If word_fixes=0
      MessageRequester("Warning!","No spaces to fix found!",#PB_MessageRequester_Warning)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR,0,"Okay.")
      RemoveWindowTimer(#WINDOW_MAIN,800)
      ProcedureReturn #False      
    EndIf
    
    
    dictionary_store_undo() ; 7/JUN/2017
    
    
    ; Fixing spaces - STEP 2 of 2    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2...0 <ESC> To abort.")
    
    word_fixes=0
    For f=0 To number_of_words-1
      text$=StringField(words$(f),1,#LF$)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        If Left(text$,1)=" " : text$=Mid(text$,2) : EndIf
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And Left(text$,1)<>" " And Right(text$,1)<>" "
      
      ; Increment fixes + fix directly in the array
      If text$<>StringField(words$(f),1,#LF$)
        word_fixes+1
        dictionary_changed=#True
        t$=text$
        If FindString(words$(f),#LF$)>0
          t$+#LF$+StringField(words$(f),2,#LF$)
        EndIf
        words$(f)=t$
      EndIf

      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2..."+Str(f)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          update_vertical_scroll_bar_dictionary()
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f    
   
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixed words:"+Str(word_fixes))  
    update_vertical_scroll_bar_dictionary()
    
        
EndProcedure




; This procedure removed the redundant code of decoding .AFF files and also has an ultrafast technique
; for finding the position of the rules, by storing in a string:
; "rule1,,position1,,rule2,,position2,," - Searching in a very small array is much faster than searching in a big .AFF
;
; Now the position is the array number - 27/APR/2018
;
; V1.0 - 17/JAN/2015
; V1.1 - 22/APR/2015
;        o Added the text while optimising the .AFF: "Optimising .AFF decode..."
;        o Optimised slightly the optimising of the .AFF
; V1.2 - 23/APR/2015
;        o Now there is a warning if we attempt to use (not supported) an .AFF with:
;          1) FLAG long  2) FLAG num
; V1.3 - 9/JUN/2015
;        o Added compatibility with:
;          1) FLAG long  2) FLAG num
;        o Optimised the decode of the .AFF
; V1.4 - 18/JAN/2016
;        o Decoding algorithm improved with minor speed gain (it now uses chr(1) INSTEAD of ",," in the ultrafast array
; V1.5 - 21/JAN/2016
;        o Simplified the code a bit
; V1.6 - 10/APR/2016
;        o Fix: Incorrect removing of spaces if spaces equals two while optimising the .AFF in the cache
;           While FindString(derived_aff_nospaces$,"  ")
;             derived_aff_nospaces$=RemoveString(derived_aff_nospaces$,"  ") >>>> derived_aff_nospaces$=ReplaceString(derived_aff_nospaces$,"  "," ") 
;           Wend
; V1.7 - 5/OCT/2016
;        o Fix: While opening a dictionary and after exiting the editor, the page length would be lost since
;               I was using the same variable for two different parts of the code.
; V1.8 - 20/MAR/2017
;        o Optimising the .AFF now shows a progress bar
;        o Simplified the .AFF optimising code
;        o Major speed gain in the .AFF optimising code (gl_ES)
; V1.9 - 4/APR/2017
;        o Support via coloured symbol for NOSUGGEST in words editor
; V2.0 - 30/MAR/2018
;        o Major speed gain in "Optimising .AFF decode cache" - up to ~30% faster
; V2.1 - 13/APR/2018
;        o Support for compressed AF
; V2.2 - 16/APR/2018
;        o NEEDAFFIX support - 16/APR/2018  
; V2.3 - 27/APR/2018
;        o Major Speed up of dictionary processing
; V2.4 - 18/MAY/2018
;        o New strategy for extra speed decoding the dictionary, but it takes more RAM.
; V2.5 - 14/JUN/2018
;        o Add/Edit dictionary: Image for NEEDAFFIX and FORBIDDENWORD
;
Procedure faster_searchable_version_of_aff()
  t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
  If derived_aff$<>t$
        
    ; Support for three types of flags
    flag_char=1 ; normal 1 chr
    If FindString(t$,"FLAG long") : flag_char=2 : EndIf ; 2 chrs
    If FindString(t$,"FLAG num") : flag_char=3 : EndIf ; numbers variable chrs  
    
FUCKTIME=ElapsedMilliseconds()   
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Optimising .AFF decode cache...")
    
    derived_aff$=t$
    derived_aff_nospaces$=t$
    ReplaceString(derived_aff_nospaces$,Chr(9)," ",#PB_String_InPlace) ; replace tabs with spaces - 13/MAR/2017
    
    While FindString(derived_aff_nospaces$,"  ")
      derived_aff_nospaces$=ReplaceString(derived_aff_nospaces$,"  "," ")       
    Wend
    derived_aff_nospaces_no_eol$=derived_aff_nospaces$+#CRLF$
    derived_aff_nospaces_no_eol$=convert_to_linux(derived_aff_nospaces_no_eol$) ; To convert to Linux Line endings    
    
    ; Convert the rules into the ultrafast search string - 17/JAN/2015 + EXTRA SPEED on 22/APR/2015
    ultrafast_aff_search$=""    
    t$=derived_aff_nospaces_no_eol$
    t_char$=t$ ; to be used in the decoding of flags while calling the procedure for faster access
    t1=1       ;Variable that stores the position of the #LF$ 
    
    
    ; Counts the total of "PFX" and "SFX" to abort if they are all processed
    sum_total_pfx_and_sfx=CountString(t$,"PFX")+CountString(t$,"SFX")
    sum_pfx_and_sfx=0 ; The current sum in the loop to abort when it matches the total
    t1=1 ; current position
    number_of_LFs=CountString(t$,#LF$) ; total number of fields to parse
    
    ; Compressed AF
    AF_compressed_number=0
    Dim AF_compressed_array$(0)
    
    
    ; Ultrafast rules
    Dim rules_of_words_total(sum_total_pfx_and_sfx)
    Dim rules_of_words_position(sum_total_pfx_and_sfx)
    Dim rules_of_words_each$(sum_total_pfx_and_sfx,0)
    Dim rules_of_words_total_direct_memory_access(0)
    rules_counter=0 ; To point to the array, instead of the address
    rules_maximum_number=0 ; To ReDim the array of the rules
    
    ; Show the progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_LFs)     
    
    AddWindowTimer(#WINDOW_MAIN,800,800)       
    For f=1 To number_of_LFs
      If sum_total_pfx_and_sfx=0 : Break : EndIf
      If sum_pfx_and_sfx=sum_total_pfx_and_sfx : Break : EndIf
        
      t_line$=StringField(t$,f,#LF$)
      pfxsfx$=StringField(t_line$,1," ")
      
      flag=#False
      If pfxsfx$="AF"
        t1+Len(t_line$)+1
        AF_compressed_number=Val(StringField(t_line$,2," "))
        Dim AF_compressed_array$(AF_compressed_number)
        For ff=1 To AF_compressed_number
          t_line$=StringField(t$,f+ff,#LF$)
          t_line$=ReplaceString(t_line$,Chr(9)," ")
          AF_compressed_array$(ff)=StringField(t_line$,2," ")
;           Debug "ff:"+Str(ff)+" AF_compressed_array$(ff)="+AF_compressed_array$(ff)
          If ff<>AF_compressed_number : t1+Len(t_line$)+1 : EndIf
        Next ff
        f+AF_compressed_number
        flag=#True
      EndIf
      
      If (pfxsfx$="PFX" Or pfxsfx$="SFX") And flag=#False
        sum_pfx_and_sfx+1
        yn$=StringField(t_line$,3," ")
        
        If yn$="Y" Or yn$="N"
          
          rules_counter+1
          
          ; For even more ultraspeed - 18/MAY/2018
          ultra_fast_position=Len(ultrafast_aff_search$)+1
          ReDim rules_of_words_total_direct_memory_access(ultra_fast_position)
          rules_of_words_total_direct_memory_access(ultra_fast_position)=rules_counter           
          
          ultrafast_aff_search$+t_line$+Chr(1)+Str(rules_counter)+Chr(1)
          rules_of_words_position(rules_counter)=t1
        
          ; Jump (ignore) the inside lines - Now it stores in an array for ultraspeed - 27/APR/2018
          number_of_lines_aff=Val(StringField(t_line$,4," "))
          rules_of_words_total(rules_counter)=number_of_lines_aff
          If number_of_lines_aff>rules_maximum_number
            rules_maximum_number=number_of_lines_aff
            ReDim rules_of_words_each$(sum_total_pfx_and_sfx,rules_maximum_number)
          EndIf
          For ff=1 To number_of_lines_aff
            ttt$=StringField(t$,f+ff,#LF$)
            rules_of_words_each$(rules_counter,ff)=ttt$
            t1+Len(ttt$)+1
          Next ff
          f+number_of_lines_aff
          
        EndIf
      EndIf
      t1+Len(t_line$)+1
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event=WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf       
            
    Next f
    
    
    ; Get the NOSUGGEST flag
    nosuggestword$=""
    nosuggestword_colour=#Black
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"NOSUGGEST")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      nosuggestword$=StringField(t_line$,2," ")
    EndIf    
    
    
    ; NEEDAFFIX support - 16/APR/2018
    needaffixword$=""
    needaffixword=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"NEEDAFFIX")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      needaffixword$=StringField(t_line$,2," ")
      needaffixword=#True
    EndIf    
    
    ; FORBIDDENWORD support - 14/JUN/2018
    forbiddenword$=""
    forbiddenword=#False
    t$=derived_aff_nospaces_no_eol$
    t=FindString(t$,"FORBIDDENWORD")
    If t>0
      t1=FindString(t$,#LF$,t)
      t_line$=Mid(t$,t,t1-t)
      forbiddenword$=StringField(t_line$,2," ")
      forbiddenword=#True
    EndIf        
    
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Okay.")
Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)  
    RemoveWindowTimer(#WINDOW_MAIN,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 21/MAR/2014    

  EndIf
  
EndProcedure




; Add the help lines to the .aff aid combobox
;
; V1.0 - 6/JAN/2017
; V1.1 - 19/MAR/2017
;        o PREFS now accept external files with rules and flags
; V1.2 - 19/MAR/2017
;        o The custom Aff aid was showing the first line (language name) in the editor combobox.
;
Procedure populate_combobox_aff_aid(image,current_aff_aid_language)
  
  t$=""
  flag=image
  
  Select image ;aff_aid_language$
      
    Case 0 ;"n/a"
      ProcedureReturn
      
      
    Case 30
      
      flag=30 ; English flag
      
      t$=""
      t$+"A # re- prefix: to do again"
      t$+#LF$
      t$+"a # mis- prefix"
      t$+#LF$
      t$+"I # in- im- il- ir- prefixes for negation"
      t$+#LF$      
      t$+"c # over- prefix: extends"
      t$+#LF$      
      t$+"U # un- prefix: reverse the action"
      t$+#LF$      
      t$+"C # de- prefix: reverse"
      t$+#LF$
      t$+"E # dis- prefix for negation"
      t$+#LF$
      t$+"F # con- prefix"
      t$+#LF$
      t$+"K # pre- prefix"
      t$+#LF$
      t$+"e # out- prefix"
      t$+#LF$
      t$+"f # under- prefix"
      t$+#LF$
      t$+"O # non- prefix"
      t$+#LF$
      t$+"4 # trans- prefix"
      t$+#LF$
      t$+"V # -ive endings for verbs"
      t$+#LF$
      t$+"v # -ively endings for verbs"
      t$+#LF$
      t$+"u # -iveness endings for verbs"
      t$+#LF$
      t$+"N # -ion ends for stem verb creating action noun"
      t$+#LF$
      t$+"n # -ion and -ions ends for stem verb creating action noun"
      t$+#LF$
      t$+"X # -ions ends for stem verb creating action noun plural"
      t$+#LF$ 
      t$+"x # -ional, -ionally ends for stem verb, simplified rule set"
      t$+#LF$
      t$+"H # adds both -th and -fold suffixes"
      t$+#LF$
      t$+"Y # -ly adverb endings (for adjectives)"
      t$+#LF$
      t$+"G # -ing endings for verbs where the stress falls on the last syllable"
      t$+#LF$
      t$+"J # plural noun form of -ings verbs, simplified rules"
      t$+#LF$
      t$+"k # -ingly endings, simplified ruleset"
      t$+#LF$
      t$+"D # regular verb past tenses and past participle with -ed"
      t$+#LF$
      t$+"d # regular verb past tenses and past participle with -ed"
      t$+#LF$
      t$+"d # regular adverbial -ing endings."
      t$+#LF$
      t$+"h # -edly to form adverb from thing. Compromise rule set."
      t$+#LF$
      t$+"i # -edness suffix. Compromise rule set"
      t$+#LF$
      t$+"T # adjectival comparatives: big bigger biggest"
      t$+#LF$
      t$+"R # -er, -ers, -er's ending for verbs creating thing or person who does"
      t$+#LF$
      t$+"r # -er, -ers, -er's ending for verbs creating thing or person who does"
      t$+#LF$
      t$+"S # noun plurals and verb conjugation (I you he we you they)"
      t$+#LF$
      t$+"P # -ness, -ness's, suffix for adjectives"
      t$+#LF$
      t$+"m # -man, -men, -man's, -men's suffixes"
      t$+#LF$
      t$+"5 # -woman, -women, -woman's, suffixes"
      t$+#LF$
      t$+"6 # -ful suffix"
      t$+#LF$
      t$+"j # -fully suffix"
      t$+#LF$      
      t$+"p # -less suffix"
      t$+#LF$
      t$+"Q # -ise, -ises, -ised, -ising"
      t$+#LF$ 
      t$+"8 # -ize, -izes, -ized, -izing"
      t$+#LF$
      t$+"q # -isation, -isations, suffixes for nouns"
      t$+#LF$
      t$+"- # -ization, -izations, suffixes for nouns"
      t$+#LF$
      t$+"s # -iser, -isers, -iser's, suffixes for nouns"
      t$+#LF$
      t$+"9 # -izer, -izers, -izer's, suffixes for nouns"
      t$+#LF$
      t$+"t # -isable, -izable, -isability, -izability, suffixes for nouns"
      t$+#LF$
      t$+"+ # -isable, -izable, -isability, -izability, suffixes for nouns"
      t$+#LF$
      t$+"M # possessive form"
      t$+#LF$   
      t$+"B # -able, -ability forms for words"
      t$+#LF$
      t$+"7 # -able, -ability forms for words where stem emphasis is NOT on last syllable"
      t$+#LF$
      t$+"g # -ability form for words where stem emphasis is NOT on last syllable"
      t$+#LF$
      t$+"l # -ably endings, simplified rules"
      t$+#LF$
      t$+"b # -ible endings, simplified rules"
      t$+#LF$
      t$+"L # -ment, -ments, -ment's suffixes"
      t$+#LF$
      t$+"Z # -y, diminutive and adjectival form, simplified rules"
      t$+#LF$
      t$+"2 # -iness, diminutive and adjectival comparison form, simplified rules"
      t$+#LF$
      t$+"z # -ily, adverbial ending for nouns where the adjective form adds y"
      t$+#LF$
      t$+"y # -ry, adjectival and noun forms"
      t$+#LF$
      t$+"O # -al, noun from verb, simplified rules"
      t$+#LF$
      t$+"o # -ally, adverb form from verb, simplified rules"
      t$+#LF$
      t$+"W # -ic, adjectival ending, simplified rule set"
      t$+#LF$
      t$+"w # -ical, adjectival ending, greatly simplified rule set"
      t$+#LF$
      t$+"1 # -ically, adverbial form ending, greatly simplified rule set"
      t$+#LF$
      t$+"3 # -ist, -ists, -ist's suffixes"
      t$+#LF$
      
      
    Default
      t$=aff_aid_language_array_rules$(current_aff_aid_language)
      t$=Mid(t$,FindString(t$,#LF$)+1) ; Fix: 19/MAR/2017
      
          
  EndSelect
  
  
  ; Don't add anything to the combobox
  If t$="" : ProcedureReturn : EndIf
  
  
  ; Add all help to the combobox
  For f=1 To CountString(t$,#LF$)
    If IsImage(flag)
      AddGadgetItem(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,-1,StringField(t$,f,#LF$),ImageID(flag))
    Else
      AddGadgetItem(#COMBOBOX_WINDOW_ADD_EDIT_AFF_AID,-1,StringField(t$,f,#LF$))
    EndIf
  Next f
  
EndProcedure




; To show the total of items selected
; 
; V1.0 - 22/DEC/2016
;
Procedure update_number_words_selected()
  
  t$="Selected:"
  If number_of_words_total_selected=0 : t$+"None" : EndIf
  If number_of_words_total_selected<>0 And number_of_words_total_selected<>number_of_words : t$+Str(number_of_words_total_selected) : EndIf
  If number_of_words_total_selected=number_of_words And number_of_words<>0 : t$+"All" : EndIf
  
  If GetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_SELECTED)<>t$ : SetGadgetText(#TEXT_PANEL_BOX_DICTIONARY_NUMBER_WORDS_SELECTED,t$) : EndIf
    
EndProcedure




; To select ALL in the array
; 
; V1.0 - 22/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_words_selected_all()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  If number_of_words>0
    For f=0 To number_of_words-1
      words_selected(f)=#True
    Next f
  EndIf
  
  number_of_words_total_selected=number_of_words
  
  update_vertical_scroll_bar_dictionary()
  
EndProcedure




; To select NONE in the array
; 
; V1.0 - 22/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_words_selected_none()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  If number_of_words>0
    For f=0 To number_of_words-1
      words_selected(f)=#False
    Next f
  EndIf  
  
  number_of_words_total_selected=0
  
  update_vertical_scroll_bar_dictionary()
  
EndProcedure




; To INVERT in the array
; 
; V1.0 - 2/MAR/2018
;
Procedure update_number_words_selected_invert()
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  count=0
  If number_of_words>0
    For f=0 To number_of_words-1
      v=words_selected(f)
      If v=#False
        v=#True
        count+1
      Else
        v=#False
      EndIf
      words_selected(f)=v
    Next f
  EndIf  
  
  number_of_words_total_selected=count
  
  update_vertical_scroll_bar_dictionary()
  
EndProcedure




; Count the number of selected words after a delete and such
;
; V1.0 - 23/DEC/2016
;
Procedure update_number_words_selected_count()
  
  number_of_words_total_selected=0
  
  If number_of_words>0
    For f=0 To number_of_words-1
      If words_selected(f)=#True : number_of_words_total_selected+1 : EndIf
    Next f
  EndIf    
  
EndProcedure




; Delete the number of selected words
;
; V1.0 - 24/DEC/2016
; V1.1 - 1/JAN/2017
;        o Fix deleting selected only when there was one item
; V1.2 - 7/JUN/2017
;        o undo/redo support
; V1.3 - 1/DEC/2017
;        o Message Requesters now have a sign. 
;
Procedure update_number_words_selected_delete()
  
  
  If number_of_words=0
    MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  
  If number_of_words_total_selected=0
    MessageRequester("Warning!","No selected words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  
  t$=""
  If number_of_words_total_selected>1 : t$="s" : EndIf
  If MessageRequester("Warning!","Really delete "+Str(number_of_words_total_selected)+" word"+t$+"?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No  
    ProcedureReturn
  EndIf
  
  
  dictionary_store_undo() ; 7/JUN/2017
  
  
  Dim buffer$(number_of_words-1)
  
  counter=0
  position=0
  For f=0 To number_of_words-1
    If words_selected(f)=#False
      buffer$(position)=words$(f)
      position+1
    EndIf
    If words_selected(f)=#True
      counter+1
      words_selected(f)=#False
    EndIf
  Next f
  
  CopyArray(buffer$(),words$())
  number_of_words-counter  
  zero_items=#False
  If number_of_words=0
    zero_items=#True
    number_of_words=1
  EndIf
  ReDim words$(number_of_words-1)
  ReDim words_selected(number_of_words-1)
  
  number_of_words_total_selected=0
  
  SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  selected_dictionary_word=-1 ; Reset the current word selected
  
  ; Fix deleting only when there was one item
  If zero_items=#True : number_of_words=0 : EndIf
  
  dictionary_changed=#True
  update_number_words()
  update_vertical_scroll_bar_dictionary()
  
EndProcedure




; To clone words from the dictionary
;
; V1.0 - 18/MAR/2017
; V1.1 - 7/JUN/2017
;        o undo/redo support
; V1.2 - 1/DEC/2017
;        o Message Requesters now have a sign. 
; V1.3 - 3/APR/2018
;        o Support for hyphenation
;
Procedure clone_word()
  
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    
    t=selected_dictionary_word
    If t=-1
      MessageRequester("Warning!","No word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf  
    
    
    dictionary_store_undo() ; 7/JUN/2017
    
    
    dictionary_changed=#True
    ReDim words$(number_of_words)        
    ReDim words_selected(number_of_words)        
    words$(number_of_words)=words$(t)
    number_of_words+1
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_words()
    update_vertical_scroll_bar_dictionary()        
    update_vertical_scroll_bar_dictionary_current_item()
    
    ; Hyphenation support - 3/APR/2018
    SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_hyphenations()
    update_vertical_scroll_bar_hyphenation()        
    update_vertical_scroll_bar_hyphenation_current_item()    
    
EndProcedure




; Disable undo + redo
;
; V1.0 - 7/JUN/2017
;
Procedure dictionary_disable_undo_redo()
  
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#True)
  Dim dictionary_undo_array(0)
  Dim dictionary_undo_array_total_selected(0)    
  dictionary_undo=#False
  dictionary_undo_number_items=0
  dictionary_undo_selected_item=-1
  dictionary_undo_items_total_selected=0
  dictionary_undo_top_of_listicongadget=0
  
  
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#True)    
  Dim dictionary_redo_array(0)
  Dim dictionary_redo_array_total_selected(0)
  dictionary_redo=#False
  dictionary_redo_number_items=0
  dictionary_redo_selected_item=-1
  dictionary_redo_items_total_selected=0
  dictionary_reddo_top_of_listicongadget=0  
  
EndProcedure




; Store undo
;
; V1.0 - 7/JUN/2017
;
Procedure dictionary_store_undo()
  Debug "STORE UNDO"
  
  ; Enable undo button and disable redo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#False)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#True)  
  
  dictionary_undo=#True
  dictionary_redo=#False
  
  Dim dictionary_undo_array$(ArraySize(words$()))
  CopyArray(words$(),dictionary_undo_array$())
  Dim dictionary_undo_array_total_selected(ArraySize(words_selected()))  
  CopyArray(words_selected(),dictionary_undo_array_total_selected())
  dictionary_undo_number_items=number_of_words
  dictionary_undo_selected_item=selected_dictionary_word
  dictionary_undo_items_total_selected=number_of_words_total_selected
  dictionary_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
EndProcedure




; Restore undo
;
; V1.0 - 7/JUN/2017
; V1.1 - 3/APR/2018
;        o Support for hyphenation
;
Procedure dictionary_restore_undo()
  Debug "RESTORE UNDO"
  
  
  ; Disable undo button and enable redo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#False)    
  
  dictionary_undo=#False
  dictionary_redo=#True
  
  Dim words$(ArraySize(dictionary_undo_array$()))
  CopyArray(dictionary_undo_array$(),words$())
  Dim words_selected(ArraySize(dictionary_undo_array_total_selected()))  
  CopyArray(dictionary_undo_array_total_selected(),words_selected())
  number_of_words=dictionary_undo_number_items
  selected_dictionary_word=dictionary_undo_selected_item
  number_of_words_total_selected=dictionary_undo_items_total_selected
;   dictionary_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_words_selected_count()
      
      
  ; Update number of selected words
  update_number_words_selected()
      
      
  ; Update number of words
  update_number_words()
  update_number_hyphenations() ; Hyphenation support - 3/APR/2018
      
      
  ; Reset dictionary
  selected_dictionary_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  update_vertical_scroll_bar_dictionary()
             
  selected_dictionary_word=dictionary_undo_selected_item
  
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,dictionary_undo_top_of_listicongadget)
  update_vertical_scroll_bar_dictionary()
  
  ; Hyphenation support - 3/APR/2018
  selected_hyphenation_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  update_vertical_scroll_bar_hyphenation()  
  
EndProcedure




; Store redo
;
; V1.0 - 7/JUN/2017
;
Procedure dictionary_store_redo()
  Debug "STORE REDO"
  
  ; Enable redo button and disable undo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#False)  
  
  dictionary_undo=#False
  dictionary_redo=#True
  
  Dim dictionary_redo_array$(ArraySize(words$()))
  CopyArray(words$(),dictionary_redo_array$())
  Dim dictionary_redo_array_total_selected(ArraySize(words_selected()))  
  CopyArray(words_selected(),dictionary_redo_array_total_selected())
  dictionary_redo_number_items=number_of_words
  dictionary_redo_selected_item=selected_dictionary_word
  dictionary_redo_items_total_selected=number_of_words_total_selected
  dictionary_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
EndProcedure




; Restore redo
;
; V1.0 - 7/JUN/2017
; V1.1 - 3/APR/2018
;        o Support for hyphenation
;
Procedure dictionary_restore_redo()
  Debug "RESTORE REDO"
  
  
  ; Disable redo button and enable undo button
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_UNDO,#False)
  DisableGadget(#BUTTON_PANEL_BOX_DICTIONARY_REDO,#True)    
  
  dictionary_undo=#True
  dictionary_redo=#False
  
  Dim words$(ArraySize(dictionary_redo_array$()))
  CopyArray(dictionary_redo_array$(),words$())
  Dim words_selected(ArraySize(dictionary_redo_array_total_selected()))  
  CopyArray(dictionary_redo_array_total_selected(),words_selected())
  number_of_words=dictionary_redo_number_items
  selected_dictionary_word=dictionary_redo_selected_item
  number_of_words_total_selected=dictionary_redo_items_total_selected
;   dictionary_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_words_selected_count()
      
      
  ; Update number of selected words
  update_number_words_selected()
      
      
  ; Update number of words
  update_number_words()
  update_number_hyphenations() ; Hyphenation support - 3/APR/2018
      
      
  ; Reset dictionary
  selected_dictionary_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  update_vertical_scroll_bar_dictionary()
  
  selected_dictionary_word=dictionary_redo_selected_item
      
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,dictionary_redo_top_of_listicongadget)
  update_vertical_scroll_bar_dictionary()
  
  ; Hyphenation support - 3/APR/2018
  selected_hyphenation_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
  update_vertical_scroll_bar_hyphenation()  
  
EndProcedure




; Move the cursor position to text position
; (more modular)
;
; V1.0 - 28/NOV/2017
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure move_to_text_position(t)
  
  CompilerIf #PB_Compiler_OS=#PB_OS_Windows
    Debug "Windows"
    SendMessage_(GadgetID(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD), #EM_SETSEL, t, t)
    SetActiveGadget(#EDITOR_GADGET_WINDOW_ADD_EDIT_WORD)
  CompilerEndIf
  CompilerIf #PB_Compiler_OS=#PB_OS_Linux
    Debug "Linux"
    MessageRequester("Warning!","Doesn't work in Linux yet.",#PB_MessageRequester_Warning)
  CompilerEndIf
  CompilerIf #PB_Compiler_OS=#PB_OS_MacOS
    Debug "Mac"
    MessageRequester("Warning!","Doesn't work in Mac yet.",#PB_MessageRequester_Warning)
  CompilerEndIf               
  
EndProcedure




; Import the places/people names into the .dic
;
; Automatise the process to save months of work
;
; V1.0 - 7/AUG/2018
;        8/AUG/2018
; V1.1 - 31/AUG/2018
;        o Added two removal strings to the "Import places/people names adding possessives" (customisable)
;
Procedure import_places_people_names()
  
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf   
    
    
    If OpenWindow(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,"Import places/people names with possessives",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)    
    
    ; EditorGadget in
    EditorGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,10,10,-15+100+400+get_extra_width(50),-10+400+get_extra_height(100))
    GadgetToolTip(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,"Paste here the full text with the names.")
    
    ; EditorGadget out
    EditorGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,10+GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+GadgetWidth(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+10,10,-15+100+400+get_extra_width(50),-10+400+get_extra_height(100))
    GadgetToolTip(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,"Output to be placed in log file.")
    
    ; CLOSE  button
    x=WindowWidth(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES)-90-10
    y=WindowHeight(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES)-25-10
    ButtonGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE,x,y,90,25,"Close")
    
    x=GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)
    y=GadgetY(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+GadgetHeight(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)+10
    
    ; Process button
    ButtonGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS,x,y,90,25,"Process")
    
    y+25+5+5+5
    
    ; Remove
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_REMOVE,x,y,100,16,"       Remove:")
    GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_REMOVE)
    x+GadgetWidth(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_REMOVE)+5-2-1
    StringGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,x,y,25,20+3,import_places_people_names_remove1$)
    x+GadgetWidth(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1)+5
    StringGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,x,y,25,20+3,import_places_people_names_remove2$)

    
    x=GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)
    
    y+25+3+2
    
    ; Possessives flag
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,x,y,100,16,"Possessives:")
    GadgetFit(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)
    StringGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,GadgetX(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+GadgetWidth(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+2,y,40+10,20+3,"/M")
    
    y+25+3+2
    
    ; Processing X of Y
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING,x,y,200,16,"Processing: 0/0")
    
    x=GadgetX(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)
    y=GadgetY(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)+GadgetHeight(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)+5+3
    
    
    ; Number of names added
    TextGadget(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL,x,y,200,16,"Found:0")
    
    y+20
    
    ; Index starting at
    CheckBoxGadget(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,x,y,100,16,"Index Starting at:")
    GadgetFit(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)
    SpinGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,x+GadgetWidth(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)+2,y,100,20+2,1,9999999,#PB_Spin_Numeric)
    SetGadgetText(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,"1")
    GadgetToolTip(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,"For log file.")
    Global index_starting_at=#True
    SetGadgetState(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,index_starting_at)
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,#PB_Shortcut_Escape,1000)    
    
    close=#False
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        close=#True
      EndIf
      
      
      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
            Case #CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING                
              index_starting_at=GetGadgetState(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)
              If index_starting_at=#True
                DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#False)
              Else
                DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#True)
              EndIf
              
            Case #STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1
              import_places_people_names_remove1$=GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1)
              
            Case #STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2  
              import_places_people_names_remove2$=GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2)
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE
              close=#True
              
            Case #BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS
              DisableGadget(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#True)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS,#True)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE,#True)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,#True)
              DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#True)
              DisableGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,#True)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,#True)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,#True)
              ClearGadgetItems(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)
              import_places_people_names_process()              
              DisableGadget(#CHECK_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#False)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESS,#False)
              DisableGadget(#BUTTON_IMPORT_PLACES_PEOPLE_NAMES_CLOSE,#False)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES,#False)
              If index_starting_at=#True : DisableGadget(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING,#False) : EndIf
              DisableGadget(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN,#False)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE1,#False)
              DisableGadget(#STRING_IMPORT_PLACES_PEOPLE_NAMES_REMOVE2,#False)              
          EndSelect
          
      EndSelect
      
    Until close=#True
    
    
    ; Dictionary support
    SetGadgetAttribute(#PANEL_BOX_DICTIONARY_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_words()
    update_vertical_scroll_bar_dictionary()  
        
    ; Hyphenation support
    SetGadgetAttribute(#LISTICONGADGET_PANEL_HYPHENATOR_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_words-1)
    update_number_hyphenations()
    update_vertical_scroll_bar_hyphenation()
        

    update_number_words()
    update_number_hyphenations()
    update_vertical_scroll_bar_dictionary_current_item()
    update_vertical_scroll_bar_hyphenation_current_item()
    
    
    CloseWindow(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES)
    DisableWindow(#WINDOW_MAIN,#False)  
    
EndProcedure




; Process the proper/places names
;
; V1.0 - 7/AUG/2018
;        8/AUG/2018
; V1.1 - 31/AUG/2018
;        o Added two removal strings to the "Import places/people names adding possessives" (customisable)
; V1.2 - 3/SEP/2018
;        o Import places/people names now also removes the chrs "+" and "%" and replaces "–" with "-"
;
Procedure import_places_people_names_process()
  
  places_people_names$=GetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_IN)
  
  places_people_names$=convert_to_linux(places_people_names$)
  
  If Right(places_people_names$,1)<>#LF$ : places_people_names$+#LF$: EndIf ; Always end with a blank line
  
  places_people_names$=ReplaceString(places_people_names$,"+"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"%"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"–","-")
  
  places_people_names$=ReplaceString(places_people_names$,","," ")
  
  places_people_names$=ReplaceString(places_people_names$,"."," ")
  
  places_people_names$=ReplaceString(places_people_names$,"&"," ")
  
  places_people_names$=ReplaceString(places_people_names$,Chr(9)," ")
  
  places_people_names$=ReplaceString(places_people_names$,"("," ")
  
  places_people_names$=ReplaceString(places_people_names$,")"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"["," ")
  
  places_people_names$=ReplaceString(places_people_names$,"]"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"{"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"}"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"/"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"*"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"?"," ")
  
  places_people_names$=ReplaceString(places_people_names$,":"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"!"," ")
  
  places_people_names$=ReplaceString(places_people_names$,";"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"|"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"--","-") ; Fix the parish that has two -'s
  
  places_people_names$=ReplaceString(places_people_names$,"0"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"1"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"2"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"3"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"4"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"5"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"6"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"7"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"8"," ")
  
  places_people_names$=ReplaceString(places_people_names$,"9"," ")
  
  If import_places_people_names_remove1$<>"" : places_people_names$=ReplaceString(places_people_names$,import_places_people_names_remove1$," ") : EndIf
  
  If import_places_people_names_remove2$<>"" : places_people_names$=ReplaceString(places_people_names$,import_places_people_names_remove2$," ") : EndIf
  
  places_people_names$=ReplaceString(places_people_names$,"  "," ")
  
  If Len(places_people_names$)=0 Or places_people_names$=" " Or places_people_names$=#LF$
    MessageRequester("Warning!","Nothing to process.",#PB_MessageRequester_Warning)
    ProcedureReturn #False
  EndIf
  
  
  possessive_flag$=GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES) ; /M
  
  undo_redo=#False ; Changes to #True if added to undo (new proper name) just used once
  
  counter=0 ; Number of names/places added
  
  AddWindowTimer(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,800,800)
  total_lines=CountString(places_people_names$,#LF$)
  For lines=1 To total_lines
    
    t$=StringField(places_people_names$,lines,#LF$)
    t$+" "
    total_spaces=CountString(t$," ")
    
    
    For import_words=1 To total_spaces
      
      name$=StringField(t$,import_words," ")
      
      If name$<>"" And Left(name$,1)=UCase(Left(name$,1)) And Left(name$,1)<>"-" And Right(name$,1)<>"-"
        
        found=#False ; So that it adds the new proper name + undo if #False
        
        For dictionary_words=0 To number_of_words-1
          
          If FindString(words$(dictionary_words),name$+possessive_flag$,1,#PB_String_CaseSensitive)
            found=#True
            Break  
          EndIf
          
          
          ; <ESC> pressed to abort
          If EventMenu()=1000
            If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
              RemoveWindowTimer(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,800) 
              
              ; redo
              If undo_redo=#True
                dictionary_restore_undo()
                undo_redo=#False
              EndIf              
              
              ProcedureReturn #False
            EndIf
          EndIf          
          
          
          ; Fix to window stop responding if we click on it while in the loop, plus update the progress bar
          event = WindowEvent()
          If event=#PB_Event_Timer And EventTimer()=800
            SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING,"Processing: "+Str(lines)+"/"+Str(total_lines))
            SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL,"Found:"+Str(counter))
          EndIf          
          
          
        Next dictionary_words
        
        ; Name not found, add it
        If found=#False
            
          ; undo
          If undo_redo=#False
            dictionary_store_undo()
            undo_redo=#True
          EndIf
            
          ReDim words$(number_of_words)        
          ReDim words_selected(number_of_words)        
          words$(number_of_words)=name$+possessive_flag$
          number_of_words+1            
        
          ; Add to the OUT EditorGadget
          index$=""
          If index_starting_at=#True : index$=Str(GetGadgetState(#SPIN_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_STARTING)+counter)+") " : EndIf
          ttt$=GetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT)+index$+name$+GetGadgetText(#STRING_IMPORT_PLACES_PEOPLE_NAMES_IN_POSSESSIVES)+#LF$
          SetGadgetText(#EDITOR_IMPORT_PLACES_PEOPLE_NAMES_OUT,ttt$)
          counter+1
            
          dictionary_changed=#True
            
        EndIf        
        
      EndIf
      
    Next import_words
    
    
  Next lines
  RemoveWindowTimer(#WINDOW_IMPORT_PLACES_PEOPLE_NAMES,800)
  
  
  SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_IN_PROCESSING,"Processing: "+Str(lines-1)+"/"+Str(total_lines)+" Done!")
  SetGadgetText(#TEXT_IMPORT_PLACES_PEOPLE_NAMES_OUT_INDEX_TOTAL,"Found:"+Str(counter))  
  
EndProcedure


; IDE Options = PureBasic 5.62 (Windows - x64)
; CursorPosition = 2844
; FirstLine = 2818
; Folding = -------
; EnableXP
; CompileSourceDirectory
; EnableUnicode